#!/usr/bin/env python

# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import jenkins
import json
import logging
import os
import re
import requests
from stevedore import driver, ExtensionManager
import xml.etree.ElementTree as ET


# Monkey patch the jenkins import
# Get Artifact
BUILD_ARTIFACT = '%(folder_url)sjob/%(short_name)s/%(number)d/artifact/%(artifact)s'


def get_build_artifact(self, name, number, artifact):
    """Get artifacts from job

    :param name: Job name, ``str``
    :param number: Build number, ``int``
    :param artifact: Artifact relative path, ``str``
    :returns: artifact to download, ``dict``
    """
    folder_url, short_name = self._get_job_folder(name)

    try:
        response = self.jenkins_open(requests.Request(
                'GET', self._build_url(BUILD_ARTIFACT, locals())))

        if response:
            return json.loads(response)
        else:
            raise jenkins.JenkinsException('job[%s] number[%d] does not exist' % (name, number))
    except requests.exceptions.HTTPError:
        raise jenkins.JenkinsException('job[%s] number[%d] does not exist' % (name, number))
    except ValueError:
        raise jenkins.JenkinsException(
                'Could not parse JSON info for job[%s] number[%d]' % (name, number))
    except jenkins.NotFoundException as e:
        # This can happen if the artifact is not found
        print("Not retrieving artifact: %s" % e)
        return None


jenkins.Jenkins.get_build_artifact = get_build_artifact


# Get Stages
BUILD_STAGES = '%(folder_url)sjob/%(short_name)s/%(number)d/wfapi/describe/'


def get_build_stages(self, name, number):
    """Get stages info from job

    :param name: Job name, ``str``
    :param number: Build number, ``int``
    :returns: dictionary of stages in the job, ``dict``
    """
    folder_url, short_name = self._get_job_folder(name)

    try:
        response = self.jenkins_open(requests.Request(
                'GET', self._build_url(BUILD_STAGES, locals())))

        if response:
            return json.loads(response)
        else:
            raise jenkins.JenkinsException('job[%s] number[%d] does not exist' % (name, number))
    except requests.exceptions.HTTPError:
        raise jenkins.JenkinsException('job[%s] number[%d] does not exist' % (name, number))
    except ValueError:
        raise jenkins.JenkinsException(
                'Could not parse JSON info for job[%s] number[%d]' % (name, number))
    except jenkins.NotFoundException as e:
        # This can happen if this isn't a stages/pipeline job
        print("Not retrieving stages: %s" % e)
        return None


jenkins.Jenkins.get_build_stages = get_build_stages
# End Monkey Patch

LOGGER = logging.getLogger(__name__)


class JenkinsCollectError(Exception):
    pass


class EsLogger(object):
    # Initialise the object
    def __init__(self, console_length, targets):
        self.data_name = type(self).__name__.lower()

        self.server = None
        self.jenkins_url = None
        self.jenkins_user = None
        self.jenkins_password = None

        self.es_job_name = None
        self.es_build_number = None
        self.build_info_fields = ['description', 'number', 'result', 'url']

        self.es_info = {}
        self.es_info['test_report'] = None
        self.es_info['stages'] = None
        self.es_info[self.data_name] = {}
        self.events = []

        self.job_xml_raw = None
        self.job_xml = None

        self.process_console_logs = []
        self.gather_build_data = []
        self.generate_events = []

        self.jenkins_url = self.get_jenkins_url()
        self.jenkins_user = self.get_jenkins_user()
        self.jenkins_password = self.get_jenkins_password()
        if self.jenkins_url:
            self.server = jenkins.Jenkins(self.jenkins_url,
                                          username=self.jenkins_user,
                                          password=self.jenkins_password)
        self.es_job_name = self.get_es_job_name()
        self.es_build_number = self.get_es_build_number()
        self.process_console_logs = self.get_process_console_logs()
        self.gather_build_data = self.get_gather_build_data()
        self.generate_events = self.get_generate_events()
        self.console_length = console_length

        self.targets = []
        for target in targets:
            self.targets.append(driver.DriverManager(namespace='es_logger.plugins.event_target',
                                                     invoke_on_load=True, name=target))
        LOGGER.info('Using targets: {}'.format(targets))

    ####################################
    # Get variables from the environment
    ####################################
    def get_jenkins_url(self):
        if not self.jenkins_url:
            return os.environ.get('JENKINS_URL')
        return self.jenkins_url

    def get_jenkins_user(self):
        if not self.jenkins_user:
            return os.environ.get('JENKINS_USER')
        return self.jenkins_user

    def get_jenkins_password(self):
        if not self.jenkins_password:
            return os.environ.get('JENKINS_PASSWORD')
        return self.jenkins_password

    def get_es_job_name(self):
        if not self.es_job_name:
            return os.environ.get('ES_JOB_NAME')
        return self.es_job_name

    def get_es_build_number(self):
        if not self.es_build_number:
            return int(os.environ.get('ES_BUILD_NUMBER', 0))
        return self.es_build_number

    def get_process_console_logs(self):
        if not self.process_console_logs:
            process_console_logs = os.environ.get('PROCESS_CONSOLE_LOGS')
            if process_console_logs in [None, '']:
                self.process_console_logs = []
            else:
                self.process_console_logs = process_console_logs.split(' ')
        return self.process_console_logs

    def get_gather_build_data(self):
        if not self.gather_build_data:
            gather_build_data = os.environ.get('GATHER_BUILD_DATA')
            if gather_build_data in [None, '']:
                self.gather_build_data = []
            else:
                self.gather_build_data = gather_build_data.split(' ')
        return self.gather_build_data

    def get_generate_events(self):
        if not self.generate_events:
            generate_events = os.environ.get('GENERATE_EVENTS')
            if generate_events in [None, '']:
                self.generate_events = []
            else:
                self.generate_events = generate_events.split(' ')
            self.generate_events.append('commit')
        return self.generate_events
    ################
    # End of getters
    ################

    # List all of the plugins we know about
    @staticmethod
    def list_plugins(data_only=False,
                     namespaces=['gather_build_data', 'console_log_processor', 'event_generator',
                                 'event_generator.console_log_events', 'event_target']):
        def print_plugin(ext):
            print("\t{}".format(ext.entry_point.name))

        plugins = {}
        for namespace in namespaces:
            plugins[namespace] = ExtensionManager(namespace='es_logger.plugins.' + namespace,
                                                  invoke_on_load=False)
        if data_only:
            ret_list = []
            for namespace in namespaces:
                ret_list += plugins[namespace].names()
            return ret_list

        for namespace in namespaces:
            print("{}:".format('es_logger.plugins.' + namespace))
            if len(plugins[namespace].names()) > 0:
                plugins[namespace].map(print_plugin)
            else:
                print("\tNone found")

    # Return the help for all of the event_target plugins loaded
    @staticmethod
    def get_event_target_plugin_help():
        ret = ''
        mgr = ExtensionManager(namespace='es_logger.plugins.event_target', invoke_on_load=False)
        for target_plugin in mgr.names():  # All known target plugins
            drv = driver.DriverManager(namespace='es_logger.plugins.event_target',
                                       invoke_on_load=False,
                                       name=target_plugin)
            ret = ret + '\n{}'.format(drv.driver.get_help_string())
        return ret

    # Get the data that we want from a build
    def get_build_data(self):
        # Add the base info we have
        self.es_info[self.data_name]['job_name'] = self.es_job_name
        self.es_info[self.data_name]['jenkins_url'] = self.jenkins_url
        self.es_info[self.data_name]['build_number'] = self.es_build_number

        try:
            # Build Info (Parameters, Status)
            self.build_info = self.server.get_build_info(self.es_job_name, self.es_build_number,
                                                         depth=0)
        except Exception as exc:
            raise JenkinsCollectError("get_build_info") from exc
        self.es_info['build_info'] = self.build_info

        try:
            self.job_xml_raw = self.server.get_job_config(self.es_job_name)
        except jenkins.JenkinsException as jenkins_err:
            LOGGER.error("JenkinsException when attempting to get job config: {}".format(
                    jenkins_err))
            self.es_info['job_config_info'] = None
            self.es_info['job_config_info_status'] = "Unable to retrieve config.xml."
        except Exception as exc:
            raise JenkinsCollectError("get_job_config") from exc

        if self.job_xml_raw is not None:
            self.job_xml = ET.fromstring(self.job_xml_raw)
            self.es_info['job_config_info'] = self.get_pipeline_job_info()
            self.es_info['job_config_info_status'] = "Retrieved config.xml."

        # Environment Variables
        try:
            self.env_vars = self.server.get_build_env_vars(self.es_job_name, self.es_build_number)
        except Exception as exc:
            raise JenkinsCollectError("get_build_env_vars") from exc
        self.es_info['env_vars'] = self.env_vars

        # Process build_info
        self.process_build_info()

        # Process the console log
        self.process_console_log()

        # Now run through any extra data gathering plugins to annotate the event
        for plugin in self.gather_build_data:
            data = driver.DriverManager(
                    namespace='es_logger.plugins.gather_build_data',
                    name=plugin,
                    invoke_on_load=True,
                    invoke_args=()
                )
            self.es_info.setdefault('build_data', {})[plugin] = data.driver.gather(self)

    def get_pipeline_job_type(self):
        pipeline_types = {"org.jenkinsci.plugins.workflow.cps.CpsFlowDefinition": "Script",
                          "org.jenkinsci.plugins.workflow.cps.CpsScmFlowDefinition": "SCM",
                          "org.jenkinsci.plugins.workflow.multibranch.SCMBinder": "Multibranch"}
        if self.job_xml is None:
            pipeline_type = None
        else:
            try:
                # Figure out what type of pipeline job this is.
                pipeline_type = pipeline_types[self.job_xml.find("./definition").attrib["class"]]
                LOGGER.debug("Pipeline Type is {}".format(pipeline_type))
            except (KeyError, AttributeError):
                LOGGER.error("Pipeline Type not found.")
                pipeline_type = "Unknown"
        return pipeline_type

    # Get some more info for a pipeline job.
    def get_pipeline_job_info(self):
        pipeline_data = {}
        if self.job_xml is None:
            pipeline_data = {}
        elif self.job_xml.tag != "flow-definition":
            pipeline_data = {"is_pipeline_job": False}
        else:
            # Store that this is a pipeline job and its type.
            pipeline_data["is_pipeline_job"] = True
            pipeline_data["pipeline_job_type"] = self.get_pipeline_job_type()

            # Check if its using git as the scm or not.
            git_scm_plugin = "hudson.plugins.git.GitSCM"
            if pipeline_data["pipeline_job_type"] in ["SCM", "Multibranch"]:
                # Both SCM types will use a Jenkinsfile
                pipeline_data["jenkinsfile"] = self.job_xml.find("./definition/scriptPath").text
                # Get the git source details from the definition or properties xml tag
                xml_tag = "./definition/" if pipeline_data["pipeline_job_type"] == "SCM" else \
                    "./properties/org.jenkinsci.plugins.workflow.multibranch." + \
                    "BranchJobProperty/branch/"
                # Get the git properties if git is in use
                if self.job_xml.find(xml_tag + "scm").attrib["class"] == git_scm_plugin:
                    pipeline_data["git_repo"] = self.job_xml.find(
                        xml_tag + "scm/userRemoteConfigs/" +
                        "hudson.plugins.git.UserRemoteConfig/url").text
                    pipeline_data["git_branch"] = self.job_xml.find(
                        xml_tag + "scm/branches/hudson.plugins.git.BranchSpec/name").text

        return pipeline_data

    # Process Build Info
    def process_build_info(self):
        for action in self.es_info['build_info']['actions']:
            # Parameters - make the data in the build_info useful
            if action.get('_class', '') in [
                    'hudson.model.ParametersAction',
                    'com.tikal.jenkins.plugins.multijob.MultiJobParametersAction']:
                for param in action['parameters']:
                    try:
                        self.es_info.setdefault(
                            'parameters', {})[param['name']] = param['value']
                    except KeyError:
                        LOGGER.debug("KeyError on {}".format(param))
                        continue
            # BuildData - parse through the data and make usable in ES
            if action.get('_class', '') in ['hudson.plugins.git.util.BuildData']:
                scm_urls = action['remoteUrls']
                for build_branch_key in action['buildsByBranchName']:
                    build_data = action['buildsByBranchName'][build_branch_key]
                    # If the build matches this number, add it to the info
                    if build_data.get('buildNumber', '') == self.es_build_number:
                        self.es_info.setdefault('build_data', {})[scm_urls[0]] = build_data
                        self.es_info['build_data'][scm_urls[0]]['scm_urls'] = scm_urls
                        self.es_info['build_data'][scm_urls[0]]['branch'] = build_branch_key
                # Clear the field that causes ES field explosion
                action['buildsByBranchName'] = "Removed by es-logger"

    # Process Console Log
    def process_console_log(self):
        try:
            self.console_log_ts = self.server.get_build_console_output(self.es_job_name,
                                                                       self.es_build_number)
        except Exception as exc:
            raise JenkinsCollectError("get_build_console_output") from exc
        # Because of timestamper 1.9+
        # https://issues.jenkins-ci.org/browse/JENKINS-48344
        # if the line starts with the timestamp, strip it
        # [2020-04-22T11:21:48.848Z] console log
        all_lines = []
        time_re = re.compile(
                r'(^\[[0-9]{4}-[0-9]{2}-[0-9]{2}' +                 # Date: '[2020-04-22'
                r'T[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3}Z\]\s)' +    # Time: 'T11:21:48.848Z] '
                r'(.*$)')                                           # Actual console log line
        for line in self.console_log_ts.splitlines():
            mo = time_re.match(line)
            if mo:
                all_lines.append(mo.group(2))
            else:
                all_lines.append(line)
        self.console_log = "\n".join(all_lines)

        # Add plugin loading and processing with an array of plugins
        # e.g. - count errors, count warnings, parse test results, etc.
        for plugin in self.process_console_logs:
            processor = driver.DriverManager(
                    namespace='es_logger.plugins.console_log_processor',
                    name=plugin,
                    invoke_on_load=True,
                    invoke_args=()
                )
            self.es_info.setdefault('console_log_plugins', {})[plugin] = processor.driver.process(
                self.console_log)

        # Max length for an indexed field is 32766, if longer, reduce
        # Default value in args parsing is 32500,
        # as when using larger values saw failures (not sure why)
        console_log_length = len(self.console_log)
        if console_log_length > self.console_length:
            self.es_info['console_log'] = self.console_log[-self.console_length:]
        else:
            self.es_info['console_log'] = self.console_log
        # Add the length of the console log
        self.es_info['console_log_length'] = console_log_length

    def get_event_info(self, fields):
        event_info = {'build_info': {}}

        if self.es_info['env_vars'] is not None:
            # XXX: Just returning self.es_info['env_vars'] is likely better...
            for field in fields:
                event_info.setdefault(
                    'env_vars', {}).setdefault('envMap', {})[field] = \
                        self.es_info['env_vars']['envMap'].get(field)

        # Add the parameters
        if self.es_info.get('parameters') is not None:
            event_info['parameters'] = self.es_info['parameters']

        # Add generic Jenkins info
        for field in self.build_info_fields:
            event_info['build_info'][field] = self.es_info['build_info'].get(field, '')
            # As we previously did this, keep doing it
            event_info[field] = event_info['build_info'][field]

        # Add the extras added by es-logger
        event_info[self.data_name] = dict(self.es_info[self.data_name])

        return event_info

    # To enable best visualisation of data,
    # allow for plugins that create an array of events to post
    def get_events(self):
        # Run through any event generator plugins
        for plugin in self.generate_events:
            event_generator = driver.DriverManager(
                    namespace='es_logger.plugins.event_generator',
                    name=plugin,
                    invoke_on_load=True,
                    invoke_args=()
                )
            gen_events = event_generator.driver.generate_events(self)

            # Get the default event info
            event_info = self.get_event_info(event_generator.driver.get_fields())
            # Add the name of the driver into the event field
            event_info[self.data_name]['event'] = plugin

            # Add the default data to each of the events we are sending in
            add_events = []
            for event in gen_events:
                new_event = {plugin: event}
                new_event.update(event_info)
                # Add the timestamp for logging when this ran
                new_event.setdefault('build_info', {})['timestamp'] = \
                    self.es_info['build_info'].get('timestamp')
                add_events.append(new_event)
            self.events += add_events

    def get_test_report(self):
        if self.es_info['test_report'] is None:
            try:
                self.es_info['test_report'] = self.server.get_build_test_report(
                    self.es_job_name, self.es_build_number)
            except Exception as exc:
                raise JenkinsCollectError("get_build_test_report") from exc
        return self.es_info['test_report']

    def get_stages(self):
        if self.es_info['stages'] is None:
            try:
                self.es_info['stages'] = self.server.get_build_stages(
                    self.es_job_name, self.es_build_number)
            except Exception as exc:
                raise JenkinsCollectError("get_build_stages") from exc
        return self.es_info['stages']

    # Dump the string
    def dump(self, json_event):
        print(json.dumps(json_event, sort_keys=True, indent=2))

    # Post the event to each target
    def post(self, json_event):
        status = 0
        for target in self.targets:
            status += target.driver.send_event(json_event)
        return status

    def finish(self):
        status = 0
        for target in self.targets:
            status += target.driver.finish_send()
        return status

    def gather_all(self):
        self.get_build_data()
        self.get_events()

    def post_all(self):
        process_events = [self.es_info] + self.events
        status = 0
        for e in process_events:
            status += self.post(e)
        status += self.finish()
        return status



#!/usr/bin/env python

# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import argparse
import es_logger
import logging
import sys


def parse_args():
    desc = '''
Read data from a completed Jenkins job and push it to a logstash instance.

Behaviour is controlled through a number of environment variables as follows:

What data to gather:
    PROCESS_CONSOLE_LOGS    Which ConsoleLogProcessor plugins to use in processing
    GATHER_BUILD_DATA       Which GatherBuildData plugins to use in processing
    GENERATE_EVENTS         Which EventGenerator plugins to use in processing

Where to gather data from:
    JENKINS_URL             The url to access Jenkins at
    JENKINS_USER            The username for Jenkins access
    JENKINS_PASSWORD        The password or API token for Jenkins access

What to gather data from:
    ES_JOB_NAME             The "Full Project Name" style job name for the job to process
    ES_BUILD_NUMBER         The build number for the job to process

Target Variables:
'''

    desc = desc + es_logger.EsLogger.get_event_target_plugin_help()

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=desc)

    output = parser.add_mutually_exclusive_group()
    output.add_argument('--no-dump', action='store_true', help='Do not dump events to the console')
    output.add_argument('--no-post', action='store_true', help='Do not post events to any targets')

    parser.add_argument(
        '-c', '--console-length', type=int, default=32500,
        help='Restrict the console length in the event to this number of characters')
    parser.add_argument(
        '-e', '--events-only', action='store_true',
        help='Do not dump or post the main job event, only events from EventGenerator plugins')
    parser.add_argument(
        '-p', '--list-plugins', action='store_true', help='List all plugins available')
    parser.add_argument(
        '-t', '--target', action='append',
        help='A target to send events to, defaults to logstash if no other is specified')

    parser.add_argument(
        '--debug', action='store_true', help='Print debug logs to console during execution')

    args = parser.parse_args()
    if args.target is None:
        args.target = ['logstash']
    return args


def configure_logging(args):
    if args.debug:
        log_level = logging.DEBUG
    else:
        log_level = logging.INFO

    logging.basicConfig(level=log_level,
                        format='%(asctime)s %(name)s %(levelname)s %(message)s')


def main():
    args = parse_args()
    configure_logging(args)

    if args.list_plugins:
        es_logger.EsLogger.list_plugins()
        sys.exit(0)

    esl = es_logger.EsLogger(console_length=args.console_length, targets=args.target)
    esl.gather_all()

    status = 0
    # Loop over all of the events to send,
    # main job in es_info, then events generated from plugins
    process_events = [esl.es_info] if not args.events_only else []
    process_events += esl.events
    for e in process_events:
        if not args.no_dump:
            esl.dump(e)
        if not args.no_post:
            status += esl.post(e)

    if not args.no_post:
        status += esl.finish()

    sys.exit(status)



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class ConsoleLogProcessor(object):
    """
    Base class for a console log processor
    """

    def __init__(self):
        super(ConsoleLogProcessor, self).__init__()

    @abc.abstractmethod
    def process(self, console_log):
        """
        Parse the console log and return a dict for ES

        :param console_log: The console log of the build
        :type console_log: str
        :returns: dict(str:?)
        """


@six.add_metaclass(abc.ABCMeta)
class GatherBuildData(object):
    """
    Base class for a build data gatherer
    """

    def __init__(self):
        super(GatherBuildData, self).__init__()

    @abc.abstractmethod
    def gather(self, esl):
        """
        Gather any extra data that is wanted to accompany the Jenkins info

        :param esl: The es-logger calling this plugin
        :type esl: object
        :returns: dict(str:?)
        """


@six.add_metaclass(abc.ABCMeta)
class EventGenerator(object):
    """
    Base class for a build data event generator
        * This will allow for the creation of an array of individual events from the data
          gathered by the EsLogger
        * Use cases include:
            * Pushing events for individual test runs
            * Pushing per-host data from a job that affects multiple hosts
    """
    # The default Jenkins fields to add to the generated events
    DEFAULT_FIELDS = [
        'BUILD_NUMBER',
        'JOB_NAME',
        'BUILD_URL'
    ]

    def __init__(self):
        super(EventGenerator, self).__init__()

    def get_fields(self):
        return self.DEFAULT_FIELDS

    @abc.abstractmethod
    def generate_events(self, esl):
        """
        Create the events to additionally push

        :param esl: The es-logger calling this plugin
        :type esl: object
        :returns: list(obj)
        """


@six.add_metaclass(abc.ABCMeta)
class ConsoleLogEventRegex(object):
    """
    Base class for a collection of regexes for use in the console_log event generator plugin
        * This allows the execution of arbitrary regexes against the console log, with each match
          being surfaced as an event from the console_log_event plugin
        * Use cases include:
            * Pushing events for docker commands
            * Pushing events to track agent in use for a job
    """

    @staticmethod
    @abc.abstractmethod
    def get_regex(self, regex_list):
        """
        Create the events to additionally push

        :param regex_list: The list to append regex's to
        :type regex_list: list
        :returns: list(obj)
        """


@six.add_metaclass(abc.ABCMeta)
class EventTarget(object):
    """
    Base class for an event target
    """

    def __init__(self):
        super(EventTarget, self).__init__()

    @staticmethod
    @abc.abstractmethod
    def get_help_string(self):
        """
        """

    @abc.abstractmethod
    def validate(self):
        """
        Validate that we have all we need to send to target

        :returns: boolean
        """

    @abc.abstractmethod
    def send_event(self, json_event):
        """
        Parse the console log and return a dict for ES

        :param json_event: The variable containing the event to send
        :type json_event: dict
        :returns: int
        """

    def finish_send(self):
        return 0



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import es_logger
import importlib.metadata
import io
from jenkins import JenkinsException, NotFoundException
import nose
from parameterized import parameterized
import requests
from stevedore import ExtensionManager
import unittest.mock
import xml.etree.ElementTree as ET


class TestEsLogger(object):

    @unittest.mock.patch.dict(
        'os.environ', {'JENKINS_URL': 'jenkins_url', 'JENKINS_USER': 'jenkins_user',
                       'JENKINS_PASSWORD': 'jenkins_password', 'ES_JOB_NAME': 'es_job_name',
                       'ES_BUILD_NUMBER': '1000', 'PROCESS_CONSOLE_LOGS':
                       'process_console_logs1 process_console_logs2 process_console_logs3',
                       'GATHER_BUILD_DATA':
                       'gather_build_data1 gather_build_data2 gather_build_data3',
                       'GENERATE_EVENTS': 'generate_events1 generate_events2 generate_events3'})
    def setup(self):
        dummy_ep = importlib.metadata.EntryPoint(
            'dummy', 'test.test_plugins:DummyEventTarget', 'es_logger.plugins.event_target')
        ExtensionManager.ENTRY_POINT_CACHE = {'es_logger.plugins.event_target': [dummy_ep]}
        self.esl = es_logger.EsLogger(1000, ['dummy'])

        self.urls = {
            'get_build_artifact': 'jenkins_url/job/job_name/1/artifact/1',
            'get_build_stages': 'jenkins_url/job/job_name/1/wfapi/describe/'}

    @unittest.mock.patch.dict(
        'os.environ', {'JENKINS_URL': 'jenkins_url', 'JENKINS_USER': 'jenkins_user',
                       'JENKINS_PASSWORD': 'jenkins_password', 'ES_JOB_NAME': 'es_job_name',
                       'ES_BUILD_NUMBER': '1000', 'PROCESS_CONSOLE_LOGS': '',
                       'GATHER_BUILD_DATA': '', 'GENERATE_EVENTS': ''})
    def test_empty_plugin_vars(self):
        dummy_ep = importlib.metadata.EntryPoint(
            'dummy', 'test.test_plugins:DummyEventTarget', 'es_logger.plugins.event_target')
        ExtensionManager.ENTRY_POINT_CACHE = {'es_logger.plugins.event_target': [dummy_ep]}
        esl = es_logger.EsLogger(1000, ['dummy'])
        nose.tools.ok_(esl.process_console_logs == [],
                       "process_console_logs {}".format(esl.process_console_logs))
        nose.tools.ok_(esl.gather_build_data == [],
                       "gather_build_data {}".format(esl.gather_build_data))
        nose.tools.ok_(esl.generate_events == ['commit'],
                       "generate_events {}".format(esl.generate_events))

    @parameterized.expand(['get_build_artifact', 'get_build_stages'])
    def test_monkey_patch(self, param):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open:
            self.esl.server.crumb = False
            mock_open.return_value = '{}'
            func = getattr(self.esl.server, param)
            if param == 'get_build_stages':
                func('job_name', 1)
            else:
                func('job_name', 1, 1)
            request = mock_open.call_args[0][0]
            print("method: {} url: {} headers: {}".format(request.method, request.url,
                                                          request.headers))
            nose.tools.ok_(request.url == self.urls[param])

    @parameterized.expand(['get_build_env_vars', 'get_build_test_report', 'get_build_artifact',
                           'get_build_stages'])
    def test_monkey_patch_missing(self, param):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open:
            mock_open.return_value = None
            self.esl.server.crumb = False
            func = getattr(self.esl.server, param)
            if param == 'get_build_artifact':
                nose.tools.assert_raises(JenkinsException,
                                         func, 'job_name', 1, 1)
            else:
                nose.tools.assert_raises(JenkinsException,
                                         func, 'job_name', 1)

    @parameterized.expand(['get_build_env_vars', 'get_build_test_report', 'get_build_artifact',
                           'get_build_stages'])
    def test_monkey_patch_http(self, param):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open:
            mock_open.side_effect = requests.exceptions.HTTPError('url', 'code', 'msg', 'hdrs',
                                                                  unittest.mock.MagicMock())
            self.esl.server.crumb = False
            func = getattr(self.esl.server, param)
            if param == 'get_build_artifact':
                nose.tools.assert_raises(JenkinsException,
                                         func, 'job_name', 1, 1)
            else:
                nose.tools.assert_raises(JenkinsException,
                                         func, 'job_name', 1)

    @parameterized.expand(['get_build_env_vars', 'get_build_test_report', 'get_build_artifact',
                           'get_build_stages'])
    def test_monkey_patch_value(self, param):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open:
            mock_open.side_effect = ValueError()
            self.esl.server.crumb = False
            func = getattr(self.esl.server, param)
            if param == 'get_build_artifact':
                nose.tools.assert_raises(JenkinsException,
                                         func, 'job_name', 1, 1)
            else:
                nose.tools.assert_raises(JenkinsException,
                                         func, 'job_name', 1)

    @parameterized.expand(['get_build_env_vars', 'get_build_test_report', 'get_build_artifact',
                           'get_build_stages'])
    def test_monkey_patch_not_found(self, param):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open:
            mock_open.side_effect = NotFoundException()
            self.esl.server.crumb = False
            func = getattr(self.esl.server, param)
            ret = 'Return not set'
            if param == 'get_build_artifact':
                ret = func('job_name', 1, 1)
            else:
                ret = func('job_name', 1)
            nose.tools.ok_(ret is None,
                           "Return not None: {}".format(ret))

    @parameterized.expand(['jenkins_url', 'jenkins_user', 'jenkins_password', 'es_job_name'])
    def test_get(self, param):
        # Recreate the esl to validate parameters aren't set
        self.esl = es_logger.EsLogger(1000, ['dummy'])
        nose.tools.ok_(getattr(self.esl, param) is None,
                       "{} not None: {}".format(param, getattr(self.esl, param)))
        getter = getattr(self.esl, 'get_' + param)
        with unittest.mock.patch.dict('os.environ', {param.upper(): param}):
            setattr(self.esl, param, getter())
            nose.tools.ok_(getter() == param,
                           "{} returned {} not {}".format(getter.__name__, getter(), param))

    def test_get_es_build_number(self):
        # Recreate the esl to validate parameters aren't set
        self.esl = es_logger.EsLogger(1000, ['dummy'])
        nose.tools.ok_(self.esl.es_build_number == 0,
                       "self.esl.es_build_number not 0: {}".format(self.esl.es_build_number))
        with unittest.mock.patch.dict('os.environ', {'ES_BUILD_NUMBER': '1000'}):
            self.esl.es_build_number = self.esl.get_es_build_number()
            nose.tools.ok_(self.esl.get_es_build_number() == 1000,
                           "{} not returning {}".format(self.esl.get_es_build_number, 1000))

    @parameterized.expand(['process_console_logs', 'gather_build_data', 'generate_events'])
    def test_get_plugin(self, param):
        if param == 'generate_events':
            base = ['commit']
        else:
            base = []
        # Recreate the esl to validate parameters aren't set
        self.esl = es_logger.EsLogger(1000, ['dummy'])
        nose.tools.ok_(getattr(self.esl, param) == base,
                       "{} not []: {}".format(param, getattr(self.esl, param)))
        setattr(self.esl, param, None)
        getter = getattr(self.esl, 'get_' + param)
        with unittest.mock.patch.dict('os.environ',
                                      {param.upper(): "{}1 {}2 {}3".format(param, param, param)}):
            plugins = getter()
            expected = [param + '1', param + '2', param + '3'] + base
            nose.tools.ok_(plugins == expected,
                           "{} returned {} not {}".format(getter.__name__, getter(), expected))

    def test_list_plugins(self):
        expected = '''es_logger.plugins.gather_build_data:
\tNone found
es_logger.plugins.console_log_processor:
\tNone found
es_logger.plugins.event_generator:
\tansible_fatal
\tansible_recap_v2
\tcommit
\tconsole_log_events
\tjunit
\tstages
es_logger.plugins.event_generator.console_log_events:
\tansible
\tes_logger
es_logger.plugins.event_target:
\tdummy
'''
        with unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            es_logger.EsLogger.list_plugins()
        nose.tools.ok_(mock_stdout.getvalue() == expected,
                       "Output was:\n{}".format(mock_stdout.getvalue()))

    def test_list_plugins_data_only(self):
        dummy_ep = importlib.metadata.EntryPoint(
            'dummy', 'test.test_plugins:DummyConsoleLogProcessor',
            'es_logger.plugins.console_log_processor')
        ExtensionManager.ENTRY_POINT_CACHE = {'es_logger.plugins.console_log_processor': [dummy_ep]}
        plugins = es_logger.EsLogger.list_plugins(True, ['console_log_processor'])
        expected = ['dummy']
        nose.tools.ok_(plugins == expected, "Output was: {}, expected {}".format(plugins, expected))

    def test_get_build_data(self):
        # Recreate the esl to validate parameters aren't set
        with unittest.mock.patch.dict(
                'os.environ', {'JENKINS_URL': 'jenkins_url', 'JENKINS_USER': 'jenkins_user',
                               'JENKINS_PASSWORD': 'jenkins_password', 'ES_JOB_NAME': 'es_job_name',
                               'ES_BUILD_NUMBER': '2', 'GATHER_BUILD_DATA': 'dummy'}):
            self.esl = es_logger.EsLogger(1000, ['dummy'])
        self.esl.es_build_number = '2'
        with unittest.mock.patch('stevedore.driver.DriverManager') as mock_driver_mgr, \
                unittest.mock.patch('jenkins.Jenkins.get_build_info') as mock_build_info, \
                unittest.mock.patch('jenkins.Jenkins.get_build_env_vars') as mock_env_vars, \
                unittest.mock.patch('jenkins.Jenkins.get_build_console_output') as mock_console, \
                unittest.mock.patch('jenkins.Jenkins.get_job_config') as mock_config:
            mock_env_vars.return_value = {'envMap': {'BUILD_NUMBER': '1',
                                                     'JOB_NAME': 'job_name',
                                                     'BUILD_URL': 'url',
                                                     'dummy': 'dummy'}}
            mock_build_info.return_value = {
                'description': 'description',
                'number': '1',
                'url': 'url',
                'actions': [{'_class': 'hudson.model.ParametersAction',
                             'parameters': [{'name': 'param', 'value': 'value'}]},
                            {'_class': 'hudson.plugins.git.util.BuildData',
                             'buildsByBranchName': {'b1': {'buildNumber': '1'},
                                                    'b2': {'buildNumber': '2'}},
                             'remoteUrls': ["repoURL"]}]}
            mock_console.return_value = 'log'

            config_xml = "<project></project>"
            mock_config.return_value = config_xml

            self.esl.get_build_data()
            mock_driver_mgr.assert_called_once()

            # Job Config recorded
            nose.tools.ok_(self.esl.es_info['job_config_info'].get("is_pipeline_job") is False,
                           "is_pipeline_job not '{}': {}".format(
                               False,
                               self.esl.es_info["job_config_info"]["is_pipeline_job"]))

            # The job_config is on of length 1.
            nose.tools.ok_(len(self.esl.es_info['job_config_info']) == 1,
                           "job_config_info len isn't '1': {}".format(
                               self.esl.es_info["job_config_info"]))

            # Console log recorded
            nose.tools.ok_(self.esl.es_info['console_log'] == 'log',
                           "console_log not 'log': {}".format(self.esl.es_info))
            # Console log length recorded
            nose.tools.ok_(self.esl.es_info['console_log_length'] == 3,
                           "console_log length not 3: {}".format(self.esl.es_info))

            # ID info pulled into eslogger namespace
            eslogger_vars = {'job_name': 'es_job_name',
                             'jenkins_url': 'jenkins_url',
                             'build_number': '2'}
            for k, v in eslogger_vars.items():
                nose.tools.ok_(self.esl.es_info['eslogger'][k] == v,
                               "{} not {}: {}".format(k, v, self.esl.es_info))

            # Parameters pulled out
            nose.tools.ok_(self.esl.es_info['parameters'].get('param') == 'value',
                           "Parameter 'param' not 'value': {}".format(self.esl.es_info))

            # Prevent ES field explosion through rewrite of builds by branch name
            nose.tools.ok_(
                self.esl.es_info['build_info']['actions'][1]['buildsByBranchName'] ==
                'Removed by es-logger',
                "buildsByBranchName not removed by es-logger: {}".format(self.esl.es_info))
            # Make sure the gather of the gather_build_data plugins was called
            nose.tools.ok_('dummy' in self.esl.es_info['build_data'].keys(),
                           "dummy not in build_data keys: {}".format(
                               self.esl.es_info))
            # SCM correctly processed
            nose.tools.ok_('repoURL' in self.esl.es_info['build_data'].keys(),
                           "repoURL not in build_data keys: {}".format(
                               self.esl.es_info['build_data']))

    def test_get_build_data_config_error(self):
        # Recreate the esl to validate parameters aren't set
        with unittest.mock.patch.dict(
                'os.environ', {'JENKINS_URL': 'jenkins_url', 'JENKINS_USER': 'jenkins_user',
                               'JENKINS_PASSWORD': 'jenkins_password', 'ES_JOB_NAME': 'es_job_name',
                               'ES_BUILD_NUMBER': '2', 'GATHER_BUILD_DATA': 'dummy'}):
            self.esl = es_logger.EsLogger(1000, ['dummy'])
        self.esl.es_build_number = '2'
        with unittest.mock.patch('stevedore.driver.DriverManager') as mock_driver_mgr, \
                unittest.mock.patch('jenkins.Jenkins.get_build_info') as mock_build_info, \
                unittest.mock.patch('jenkins.Jenkins.get_build_env_vars') as mock_env_vars, \
                unittest.mock.patch('jenkins.Jenkins.get_build_console_output') as mock_console, \
                unittest.mock.patch('jenkins.Jenkins.get_job_config') as mock_config:
            mock_env_vars.return_value = {'envMap': {'BUILD_NUMBER': '1',
                                                     'JOB_NAME': 'job_name',
                                                     'BUILD_URL': 'url',
                                                     'dummy': 'dummy'}}
            mock_build_info.return_value = {
                'description': 'description',
                'number': '1',
                'url': 'url',
                'actions': [{'_class': 'hudson.model.ParametersAction',
                             'parameters': [{'name': 'param', 'value': 'value'}]},
                            {'_class': 'hudson.plugins.git.util.BuildData',
                             'buildsByBranchName': {'b1': {'buildNumber': '1'},
                                                    'b2': {'buildNumber': '2'}},
                             'remoteUrls': ["repoURL"]}]}
            mock_console.return_value = 'log'
            mock_config.side_effect = JenkinsException("Error from Jenkins Api")

            self.esl.get_build_data()
            mock_driver_mgr.assert_called_once()

            # Job Config not recorded
            expected_error_msg = "Unable to retrieve config.xml."
            nose.tools.ok_(self.esl.es_info['job_config_info'] is None,
                           "'job_config_info' value {} not '{}'".format(
                               self.esl.es_info['job_config_info'], None))
            nose.tools.ok_(self.esl.es_info['job_config_info_status'] == expected_error_msg,
                           "'job_config_info_status' value {} not '{}'".format(
                               self.esl.es_info['job_config_info_status'], expected_error_msg))

            # Console log recorded
            nose.tools.ok_(self.esl.es_info['console_log'] == 'log',
                           "console_log not 'log': {}".format(self.esl.es_info))
            # Parameters pulled out
            nose.tools.ok_(self.esl.es_info['parameters'].get('param') == 'value',
                           "Parameter 'param' not 'value': {}".format(self.esl.es_info))

            # Prevent ES field explosion through rewrite of builds by branch name
            nose.tools.ok_(
                self.esl.es_info['build_info']['actions'][1]['buildsByBranchName'] ==
                'Removed by es-logger',
                "buildsByBranchName not removed by es-logger: {}".format(self.esl.es_info))
            # Make sure the gather of the gather_build_data plugins was called
            nose.tools.ok_('dummy' in self.esl.es_info['build_data'].keys(),
                           "dummy not in build_data keys: {}".format(
                               self.esl.es_info))
            # SCM correctly processed
            nose.tools.ok_('repoURL' in self.esl.es_info['build_data'].keys(),
                           "repoURL not in build_data keys: {}".format(
                               self.esl.es_info['build_data']))

    def test_get_pipeline_job_type_script(self):
        script = "org.jenkinsci.plugins.workflow.cps.CpsFlowDefinition"

        mock_job_xml = unittest.mock.MagicMock()
        mock_job_xml.find.return_value.attrib.__getitem__.return_value = script
        self.esl.job_xml = mock_job_xml

        test_result = self.esl.get_pipeline_job_type()

        mock_job_xml.find.assert_called_with("./definition")
        nose.tools.ok_(test_result == "Script",
                       "get_pipeline_job_type, returned: {}, expected {}".format(
                           test_result, "Script"))

    def test_get_pipeline_job_type_scm(self):
        scm = "org.jenkinsci.plugins.workflow.cps.CpsScmFlowDefinition"
        mock_job_xml = unittest.mock.MagicMock()
        mock_job_xml.find.return_value.attrib.__getitem__.return_value = scm
        self.esl.job_xml = mock_job_xml

        test_result = self.esl.get_pipeline_job_type()

        mock_job_xml.find.assert_called_with("./definition")
        nose.tools.ok_(test_result == "SCM",
                       "get_pipeline_job_type, returned: {}, expected {}".format(
                           test_result, "SCM"))

    def test_get_pipeline_job_type_multibranch(self):
        multibranch = "org.jenkinsci.plugins.workflow.multibranch.SCMBinder"
        mock_job_xml = unittest.mock.MagicMock()
        mock_job_xml.find.return_value.attrib.__getitem__.return_value = multibranch
        self.esl.job_xml = mock_job_xml

        test_result = self.esl.get_pipeline_job_type()

        mock_job_xml.find.assert_called_with("./definition")
        nose.tools.ok_(test_result == "Multibranch",
                       "get_pipeline_job_type, returned: {}, expected {}".format(
                           test_result, "Multibranch"))

    def test_get_pipeline_job_type_invalid(self):
        mock_job_xml = unittest.mock.MagicMock()
        mock_job_xml.find.return_value.attrib.__getitem__.return_value = "blah"
        self.esl.job_xml = mock_job_xml

        test_result = self.esl.get_pipeline_job_type()
        mock_job_xml.find.assert_called_with("./definition")
        nose.tools.ok_(test_result == "Unknown",
                       "get_pipeline_job_type, returned: {}, expected {}".format(
                           test_result, "Unknown"))

    def test_get_pipeline_job_type_not_pipeline_job(self):
        self.esl.job_xml = "Not a pipeline job"

        test_result = self.esl.get_pipeline_job_type()
        nose.tools.ok_(test_result == "Unknown",
                       "get_pipeline_job_type, returned: {}, expected {}".format(
                           test_result, "Unknown"))

    def test_get_pipeline_job_type_config_xml_is_none(self):
        test_result = self.esl.get_pipeline_job_type()
        nose.tools.ok_(test_result is None,
                       "get_pipeline_job_type, returned: {}, expected {}".format(
                           test_result, None))

    def test_get_pipeline_job_info_not_pipeline(self):
        test_mock = unittest.mock.Mock()
        test_mock.tag = "Random tag"
        self.esl.job_xml = test_mock

        test_result = self.esl.get_pipeline_job_info()

        nose.tools.ok_(test_result == {"is_pipeline_job": False},
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, {"is_pipeline_job": False}))

    def test_get_pipeline_job_info_unknown_pipeline(self):
        unknown_pipeline_xml = """
            <flow-definition plugin="workflow-job@2.29">
                <definition class="What is this?"></definition>
            </flow-definition>"""
        self.esl.job_xml = ET.fromstring(unknown_pipeline_xml)

        test_result = self.esl.get_pipeline_job_info()

        expected_result = {"is_pipeline_job": True,
                           "pipeline_job_type": "Unknown"}

        nose.tools.ok_(test_result == expected_result,
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, expected_result))

    def test_get_pipeline_job_info_pipeline_script(self):
        pipeline_script_xml = """
            <flow-definition plugin="workflow-job@2.29">
                <definition class="org.jenkinsci.plugins.workflow.cps.CpsFlowDefinition"
                        plugin="workflow-cps@2.46">
                </definition>
            </flow-definition>"""
        self.esl.job_xml = ET.fromstring(pipeline_script_xml)

        test_result = self.esl.get_pipeline_job_info()
        expected_result = {"is_pipeline_job": True,
                           "pipeline_job_type": "Script"}

        nose.tools.ok_(test_result == expected_result,
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, expected_result))

    def test_get_pipeline_job_info_pipeline_scm_not_git(self):
        pipeline_scm_not_git_xml = """
            <flow-definition plugin="workflow-job@2.29">
                <definition class="org.jenkinsci.plugins.workflow.cps.CpsScmFlowDefinition"
                        plugin="workflow-cps@2.46">
                    <scm class="Not Git" plugin="something else">
                    </scm>
                    <scriptPath>JenkinsfileName</scriptPath>
                </definition>
            </flow-definition>"""
        self.esl.job_xml = ET.fromstring(pipeline_scm_not_git_xml)

        test_result = self.esl.get_pipeline_job_info()
        expected_result = {"is_pipeline_job": True,
                           "pipeline_job_type": "SCM",
                           "jenkinsfile": "JenkinsfileName"}

        nose.tools.ok_(test_result == expected_result,
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, expected_result))

    def test_get_pipeline_job_info_pipeline_scm_git(self):
        pipeline_scm_git_xml = """
            <flow-definition plugin="workflow-job@2.29">
                <definition class="org.jenkinsci.plugins.workflow.cps.CpsScmFlowDefinition"
                        plugin="workflow-cps@2.46">
                    <scm class="hudson.plugins.git.GitSCM" plugin="git@3.9.1">
                        <userRemoteConfigs>
                            <hudson.plugins.git.UserRemoteConfig>
                                <url>git_repo_addr</url>
                            </hudson.plugins.git.UserRemoteConfig>
                        </userRemoteConfigs>
                        <branches>
                            <hudson.plugins.git.BranchSpec>
                                <name>branch_string</name>
                            </hudson.plugins.git.BranchSpec>
                        </branches>
                    </scm>
                    <scriptPath>JenkinsfileName</scriptPath>
                </definition>
            </flow-definition>"""
        self.esl.job_xml = ET.fromstring(pipeline_scm_git_xml)

        test_result = self.esl.get_pipeline_job_info()
        expected_result = {"is_pipeline_job": True,
                           "pipeline_job_type": "SCM",
                           "jenkinsfile": "JenkinsfileName",
                           "git_repo": "git_repo_addr",
                           "git_branch": "branch_string"}

        nose.tools.ok_(test_result == expected_result,
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, expected_result))

    def test_get_pipeline_job_info_pipeline_multibranch_no_git(self):
        pipeline_multibranch_not_git_xml = """
            <flow-definition plugin="workflow-job@2.29">
                <properties>
                    <org.jenkinsci.plugins.workflow.multibranch.BranchJobProperty
                                   plugin="workflow-multibranch@2.20">
                        <branch plugin="branch-api@2.0.20.1">
                            <scm class="Not Git" plugin="something else">
                            </scm>
                        </branch>
                    </org.jenkinsci.plugins.workflow.multibranch.BranchJobProperty>
                </properties>
                <definition class="org.jenkinsci.plugins.workflow.multibranch.SCMBinder"
                            plugin="workflow-multibranch@2.20">
                    <scriptPath>JenkinsfileName</scriptPath>
                </definition>
            </flow-definition>"""
        self.esl.job_xml = ET.fromstring(pipeline_multibranch_not_git_xml)

        test_result = self.esl.get_pipeline_job_info()
        expected_result = {"is_pipeline_job": True,
                           "pipeline_job_type": "Multibranch",
                           "jenkinsfile": "JenkinsfileName"}

        nose.tools.ok_(test_result == expected_result,
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, expected_result))

    def test_get_pipeline_job_info_pipeline_multibranch_git(self):
        pipeline_multibranch_git_xml = """
            <flow-definition plugin="workflow-job@2.29">
                <properties>
                    <org.jenkinsci.plugins.workflow.multibranch.BranchJobProperty
                                   plugin="workflow-multibranch@2.20">
                        <branch plugin="branch-api@2.0.20.1">
                            <scm class="hudson.plugins.git.GitSCM" plugin="git@3.9.3">
                                <userRemoteConfigs>
                                    <hudson.plugins.git.UserRemoteConfig>
                                        <url>git_repo_addr</url>
                                    </hudson.plugins.git.UserRemoteConfig>
                                </userRemoteConfigs>
                                <branches class="singleton-list">
                                    <hudson.plugins.git.BranchSpec>
                                        <name>branch_string</name>
                                    </hudson.plugins.git.BranchSpec>
                                </branches>
                            </scm>
                        </branch>
                    </org.jenkinsci.plugins.workflow.multibranch.BranchJobProperty>
                </properties>
                <definition class="org.jenkinsci.plugins.workflow.multibranch.SCMBinder"
                            plugin="workflow-multibranch@2.20">
                    <scriptPath>JenkinsfileName</scriptPath>
                </definition>
            </flow-definition>"""
        self.esl.job_xml = ET.fromstring(pipeline_multibranch_git_xml)

        test_result = self.esl.get_pipeline_job_info()
        expected_result = {"is_pipeline_job": True,
                           "pipeline_job_type": "Multibranch",
                           "jenkinsfile": "JenkinsfileName",
                           "git_repo": "git_repo_addr",
                           "git_branch": "branch_string"}

        nose.tools.ok_(test_result == expected_result,
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, expected_result))

    def test_get_pipeline_job_info_config_xml_is_none(self):
        test_result = self.esl.get_pipeline_job_info()
        nose.tools.ok_(test_result == {},
                       "get_pipeline_job_info, returned: {}, expected {}".format(
                           test_result, {}))

    def test_process_build_info_actions_no_key_error(self):
        self.esl.es_info = {'build_info': {'actions': [
            {'_class': 'hudson.model.ParametersAction', 'parameters': [{'name': 'param'}]}]}}
        self.esl.process_build_info()

    # This is tested through get of the build data above
    def test_process_build_info(self):
        pass

    def test_process_console_log(self):
        self.esl.process_console_logs = ['dummy']
        with unittest.mock.patch('stevedore.driver.DriverManager') as mock_driver_mgr:
            with unittest.mock.patch('jenkins.Jenkins.get_build_console_output') as mock_get:
                mock_get.return_value = 'log\n[2020-04-22T11:21:48.848Z] log2 words\nlog3'
                self.esl.process_console_log()
                mock_driver_mgr.assert_called_once()
                self.esl.console_length = 1
                mock_get.return_value = 'log\nlog2 words\nlog3'
                self.esl.process_console_log()
                nose.tools.ok_(len(self.esl.es_info['console_log']) == 1,
                               "Console log length not 1: {}".format(
                                    self.esl.es_info['console_log']))

    # Tested in get_events
    def test_get_event_info(self):
        pass

    def test_get_events(self):
        self.esl.generate_events = ['dummy']
        self.esl.es_info['env_vars'] = {'envMap': {'BUILD_NUMBER': '1',
                                                   'JOB_NAME': 'job_name',
                                                   'BUILD_URL': 'url',
                                                   'dummy': 'dummy'}}
        self.esl.es_info['build_info'] = {'description': 'description',
                                          'number': '1',
                                          'result': 'SUCCESS',
                                          'url': 'url'}
        self.esl.es_info['parameters'] = {'parameter': 'parameter'}
        self.esl.es_info[self.esl.data_name]['job_name'] = 'job_name'
        self.esl.es_info[self.esl.data_name]['jenkins_url'] = 'url'
        with unittest.mock.patch('stevedore.driver.DriverManager') as mock_driver_mgr:
            mock_driver_mgr().driver.get_fields.return_value = 'dummy'
            mock_driver_mgr().driver.generate_events.return_value = [{'DummyEventGenerator': 1},
                                                                     {'DummyEventGenerator': 2}]
            self.esl.get_events()
            nose.tools.ok_(mock_driver_mgr.call_count == 3,
                           "Incorrect call count: {}".format(mock_driver_mgr.call_count))
            nose.tools.ok_(len(self.esl.events) == 2,
                           "Incorrect number of events: {}".format(self.esl.events))
            nose.tools.ok_('timestamp' in self.esl.events[0]['build_info'].keys(),
                           "No timestamp in event: {}".format(
                                self.esl.events[0]['build_info'].keys()))
            for field in self.esl.build_info_fields:
                for count in range(2):
                    nose.tools.ok_(self.esl.events[count]['build_info'][field] ==
                                   self.esl.es_info['build_info'][field],
                                   "Incorrect {} in event {} build_info: {}".format(
                                        self.esl.events[count]['build_info'][field], count,
                                        self.esl.es_info['build_info'][field]))
                    nose.tools.ok_(self.esl.events[count]['build_info'][field] ==
                                   self.esl.events[count][field],
                                   "Incorrect {} in event {} build_info: {}".format(
                                        self.esl.events[count]['build_info'][field], count,
                                        self.esl.events[count][field]))

    def test_gather_all(self):
        with unittest.mock.patch('es_logger.EsLogger.get_build_data') as mock_get_build_data, \
                unittest.mock.patch('es_logger.EsLogger.get_events') as mock_get_events:
            self.esl.gather_all()
            mock_get_build_data.assert_called_once()
            mock_get_events.assert_called_once()

    def test_post_all(self):
        with unittest.mock.patch('es_logger.EsLogger.post') as mock_post, \
                unittest.mock.patch('es_logger.EsLogger.finish') as mock_finish:
            self.esl.post_all()
            mock_post.assert_called_once()
            mock_finish.assert_called_once()

    def test_get_test_report(self):
        with unittest.mock.patch('jenkins.Jenkins.get_build_test_report') as mock_get:
            self.esl.get_test_report()
            mock_get.assert_called_once()
        nose.tools.ok_(self.esl.es_info['test_report'] is not None,
                       "Test report is None: {}".format(self.esl.es_info['test_report']))

    def test_get_stages(self):
        with unittest.mock.patch('jenkins.Jenkins.get_build_stages') as mock_get:
            self.esl.get_stages()
            mock_get.assert_called_once()
        nose.tools.ok_(self.esl.es_info['stages'] is not None,
                       "Stages is None: {}".format(self.esl.es_info['stages']))

    def test_dump(self):
        with unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            self.esl.dump({"event": "event"})
        print(mock_stdout.getvalue())
        nose.tools.ok_(mock_stdout.getvalue() == '{\n  "event": "event"\n}\n',
                       "Incorrect output:\n{}".format(mock_stdout.getvalue()))

    def test_post(self):
        status = self.esl.post({"es-logger": True})
        nose.tools.ok_(status == 0)

    def test_post_bad(self):
        status = self.esl.post(None)
        nose.tools.ok_(status == 1)

    def test_finish(self):
        status = self.esl.finish()
        nose.tools.ok_(status == 0)

    def test_finish_bad(self):
        mock_target = unittest.mock.MagicMock()
        mock_target.driver.finish_send.return_value = 1
        self.esl.targets = [mock_target, mock_target]
        status = self.esl.finish()
        nose.tools.ok_(status == 2)

    @parameterized.expand(['get_build_data', 'process_console_log', 'get_test_report',
                           'get_stages'])
    def test_exception_wraps(self, param):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open:
            mock_open.return_value = None
            self.esl.server.crumb = False
            func = getattr(self.esl, param)
            nose.tools.assert_raises(es_logger.JenkinsCollectError, func)

    # Needs a slightly different flow because 2nd call in the get_build_data function
    # Although the same outcome, ensures coverage, so is testing the right spot
    def test_exception_wraps_get_build_data(self):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open, \
                unittest.mock.patch('jenkins.Jenkins.get_build_info') as mock_build_info:
            mock_build_info.return_value = {}
            mock_open.return_value = None
            self.esl.server.crumb = False
            nose.tools.assert_raises(es_logger.JenkinsCollectError, self.esl.get_build_data)

    # Needs a slightly different flow because get_job_config needs to throw a non-Jenkins error
    # Although the same outcome, ensures coverage, so is testing the right spot
    def test_exception_wraps_get_job_config(self):
        with unittest.mock.patch('es_logger.jenkins.Jenkins.jenkins_open') as mock_open, \
                unittest.mock.patch('jenkins.Jenkins.get_build_info') as mock_build_info:
            mock_build_info.return_value = {}
            mock_open.side_effect = Exception("Wrap me")
            self.esl.server.crumb = False
            nose.tools.assert_raises(es_logger.JenkinsCollectError, self.esl.get_build_data)



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import es_logger
from .event_generator_base import TestEventGenerator
import nose
import unittest.mock


class TestAnsibleRecapEvent(object):
    @nose.tools.raises(ValueError)
    def test_ansible_bad_hosts(self):
        eg = es_logger.plugins.ansible.AnsibleRecapEvent()
        fields = eg.get_fields()
        nose.tools.ok_(fields == es_logger.interface.EventGenerator.DEFAULT_FIELDS)
        esl = unittest.mock.MagicMock()
        esl.console_log = '''
+ ansible-playbook command

PLAY [Play 1] ******************************************************************

TASK [Task 1] ******************************************************************
Sunday 15 April 2018  11:36:41 +0000 (0:00:00.096)       0:00:00.096 **********
ok: [host1 -> localhost]

PLAY RECAP *********************************************************************

host1 : ok=13  bad changed=3   unreachable=1    failed=0

Sunday 15 April 2018  11:40:35 +0000 (0:00:03.299)       1:29:59.000 **********
===============================================================================
plays-in-role-1 : The first task performed ---------------------------- 161.34s
'''
        eg.generate_events(esl)

    def test_ansible_strip_ansi_colour(self):
        eg = es_logger.plugins.ansible.AnsibleRecapEvent()
        fields = eg.get_fields()
        nose.tools.ok_(fields == es_logger.interface.EventGenerator.DEFAULT_FIELDS)
        esl = unittest.mock.MagicMock()
        esl.console_log = '''
+ ansible-playbook command

PLAY [Play 1] ******************************************************************

TASK [Task 1] ******************************************************************
Sunday 15 April 2018  11:36:41 +0000 (0:00:00.096)       0:00:00.096 **********
ok: [host1 -> localhost]

PLAY RECAP *********************************************************************
host1 : \x1b[0;32mok=13  \x1b[0m \x1b[0;33mchanged=3   \x1b[0m unreachable=1    failed=0

Sunday 15 April 2018  11:40:35 +0000 (0:00:03.299)       1:29:59.000 **********
===============================================================================
plays-in-role-1 : The first task performed ---------------------------- 161.34s
'''
        events = eg.generate_events(esl)
        nose.tools.ok_(len(events) == 2,
                       "Wrong number of events returned ({}): {}".format(len(events), events))
        results = [
            {'play': 'Play 1', 'host': 'host1', 'ok': 13, 'changed': 3, 'unreachable': 1,
             'failed': 0},
            {'play': 'Play 1', 'total': 5399.0, 'time_percentage': 2.9883311724393407,
             'description': 'plays-in-role-1 : The first task performed', 'time': 161.34}]

        for idx, event in enumerate(events):
            nose.tools.ok_(event == results[idx],
                           "Bad event[{}] returned: {}".format(idx, events))

    def test_ansible_recap_event(self):
        eg = es_logger.plugins.ansible.AnsibleRecapEvent()
        fields = eg.get_fields()
        nose.tools.ok_(fields == es_logger.interface.EventGenerator.DEFAULT_FIELDS)
        esl = unittest.mock.MagicMock()
        esl.console_log = '''
+ ansible-playbook command

PLAY [Play 1] ******************************************************************

TASK [Task 1] ******************************************************************
Sunday 15 April 2018  11:36:41 +0000 (0:00:00.096)       0:00:00.096 **********
ok: [host1 -> localhost]

PLAY RECAP *********************************************************************
host1 : ok=1   changed=2    unreachable=3    failed=4
host1 : ok=8   changed=7    unreachable=6    failed=5

Sunday 15 April 2018  11:40:35 +0000 (0:00:03.299)       1:29:59.000 **********
===============================================================================
plays-in-role-1 : The first task performed ---------------------------- 161.34s
plays-in-role-2 : The last task performed with a long name leaving only 3 dashes --- 0.03s
plays-in-role-3 : The last task performed with a long name leaving only 2 spaces  1.03s
+ ansible-playbook command
 [WARNING]: Found variable using reserved name: hosts


PLAY [Play 2] *********************

TASK [role2 : Perform task 1] *****
changed: [host2]

TASK [role2 : Perform task 2] *****
changed: [host2]

PLAY [Play 3] *********************************************
skipping: no hosts matched

PLAY RECAP *********************************************************************
host2 : ok=4   changed=3    unreachable=2    failed=1
host3 : ok=5   changed=6    unreachable=7    failed=8

TASK: role2 : task 1 --- 161.27s (not verified)
TASK: role2 : task 2 ----------------- 0.29s (not verified)

Total -------------------------------------------------- 259.72s (4 min 20 sec)

Finished: SUCCESS
'''
        events = eg.generate_events(esl)
        nose.tools.ok_(len(events) == 9,
                       "Wrong number of events returned ({}): {}".format(len(events), events))
        results = [
            {'play': 'Play 2', 'host': 'host2', 'ok': 4, 'changed': 3, 'unreachable': 2,
             'failed': 1},
            {'play': 'Play 2', 'host': 'host3', 'ok': 5, 'changed': 6, 'unreachable': 7,
             'failed': 8},
            {'play': 'Play 2', 'total': 259.72, 'time_percentage': 62.09379331587863,
             'description': 'TASK: role2 : task 1', 'time': 161.27},
            {'play': 'Play 2', 'total': 259.72, 'time_percentage': 0.11165870937933156,
             'description': 'TASK: role2 : task 2', 'time': 0.29},
            {'play': 'Play 1', 'host': 'host1', 'ok': 1, 'changed': 2, 'unreachable': 3,
             'failed': 4},
            {'play': 'Play 1', 'host': 'host1', 'ok': 8, 'changed': 7, 'unreachable': 6,
             'failed': 5},
            {'play': 'Play 1', 'total': 5399.0, 'time_percentage': 2.9883311724393407,
             'description': 'plays-in-role-1 : The first task performed', 'time': 161.34},
            {'play': 'Play 1', 'total': 5399.0, 'time_percentage': 0.0005556584552694943,
             'description':
             'plays-in-role-2 : The last task performed with a long name leaving only 3 dashes',
             'time': 0.03},
            {'play': 'Play 1', 'total': 5399.0, 'time_percentage': 0.01907760696425264,
             'description':
             'plays-in-role-3 : The last task performed with a long name leaving only 2 spaces',
             'time': 1.03}]

        for idx, event in enumerate(events):
            nose.tools.ok_(event == results[idx],
                           "Bad event[{}] returned: {}".format(idx, events))


class TestAnsibleFatalGenerator(TestEventGenerator):
    def test_AnsibleFatalGenerator(self):
        p = es_logger.plugins.ansible.AnsibleFatalGenerator()
        self.expected_fields_check(p.get_fields(), [])
        self.esl.console_log = '''
TASK [ansible-task : Task Description] *****************************************
Thursday 01 March 2018  13:24:49 +0000 (0:00:05.686)       0:00:05.901 ********
fatal: [ansible_hostname]: FAILED! => {"failed": true, "msg": "Error message\n"}
        to retry, use: --limit @/ansible_root/ansible/playbook.retry

TASK [ansible-task : Task Description] *****************************************
Thursday 01 March 2018  13:24:49 +0000 (0:00:05.686)       0:00:05.901 ********
fatal: [ansible_hostname]: UNREACHABLE! => {"changed": false, "msg": "Error", "unreachable": true}
        to retry, use: --limit @/ansible_root/ansible/playbook.retry

PLAY RECAP *********************************************************************
'''
        ret = p.generate_events(self.esl)
        self.return_length_check(ret, 2)
        nose.tools.ok_(ret[0]['hostname'] == 'ansible_hostname',
                       "Hostname is not 'ansible_hostname' {}".format(ret))
        nose.tools.ok_('failed' in ret[0]['data'].keys(),
                       "Data doesn't have 'failed' key {}".format(ret))
        nose.tools.ok_('msg' in ret[0]['data'].keys(),
                       "Data doesn't have 'msg' key {}".format(ret))

    def test_AnsibleFatalGenerator_bad_json(self):
        p = es_logger.plugins.ansible.AnsibleFatalGenerator()
        self.expected_fields_check(p.get_fields(), [])
        self.esl.console_log = '''
TASK [ansible-task : Task Description] *****************************************
Thursday 01 March 2018  13:24:49 +0000 (0:00:05.686)       0:00:05.901 ********
fatal: [ansible_hostname]: FAILED! => {"failed": true, "msg": "Error message\n"}}
        to retry, use: --limit @/ansible_root/ansible/playbook.retry

TASK [ansible-task : Task Description] *****************************************
Thursday 01 March 2018  13:24:49 +0000 (0:00:05.686)       0:00:05.901 ********
fatal: [ansible_hostname]: UNREACHABLE! => {"changed": false, "msg": "Error", "unreachable": true}}
        to retry, use: --limit @/ansible_root/ansible/playbook.retry

PLAY RECAP *********************************************************************
'''
        ret = p.generate_events(self.esl)
        self.return_length_check(ret, 2)
        nose.tools.ok_(ret[0]['hostname'] == 'ansible_hostname',
                       "Hostname is not 'ansible_hostname' {}".format(ret))
        nose.tools.ok_(ret[1]['hostname'] == 'ansible_hostname',
                       "Hostname is not 'ansible_hostname' {}".format(ret))
        nose.tools.ok_('bad_data' in ret[0].keys(),
                       "No 'bad_data' key: {}".format(ret))



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import es_logger
import nose
import unittest.mock


class TestJUnitEvent(object):
    def test_junit_fields(self):
        eg = es_logger.plugins.junit.JUnitEvent()
        fields = eg.get_fields()
        nose.tools.ok_(fields != es_logger.interface.EventGenerator.DEFAULT_FIELDS)

    def test_no_test_report(self):
        eg = es_logger.plugins.junit.JUnitEvent()
        esl = unittest.mock.MagicMock()
        esl.get_test_report.return_value = None
        events = eg.generate_events(esl)
        nose.tools.ok_(len(events) == 0)

    def test_with_test_report(self):
        eg = es_logger.plugins.junit.JUnitEvent()
        esl = unittest.mock.MagicMock()
        esl.get_test_report.return_value = {
            "_class": "hudson.tasks.junit.TestResult", "testActions": [], "duration": 4550245900,
            "empty": False, "failCount": 2, "passCount": 1, "skipCount": 1, "suites": [{
                "cases": [{
                    "testActions": [], "age": 0, "className": "com.test.one.Test",
                    "duration": 0.638, "errorDetails": None, "errorStackTrace": None,
                    "failedSince": 0, "name": "testone", "skipped": False,
                    "skippedMessage": None, "status": "PASSED", "stderr": "", "stdout": ""
                }, {
                    "testActions": [], "age": 0, "className": "com.test.two.Test",
                    "duration": 0.053, "errorDetails": None, "errorStackTrace": None,
                    "failedSince": 0, "name": "testtwo", "skipped": False, "skippedMessage": None,
                    "status": "SKIPPED", "stderr": "", "stdout": ""
                }, {
                    "testActions": [], "age": 0, "className": "com.test.three.Test",
                    "duration": 0.053, "errorDetails": 'Error Detail', "errorStackTrace": None,
                    "failedSince": 0, "name": "testthree", "skipped": False,
                    "skippedMessage": None, "status": "FAILED", "stderr": "", "stdout": ""
                }],
                "duration": 1516748670, "id": None, "name": "com.suite.one.Suite", "stderr": "",
                "stdout": "", "timestamp": "1970-01-01T00:00:00"
            }, {
                "cases": [{
                    "testActions": [], "age": 0, "className": "com.test.three.Test",
                    "duration": 0.551, "errorDetails": None, "errorStackTrace": None,
                    "failedSince": 0, "name": "testfour", "skipped": False, "skippedMessage": None,
                    "status": "REGRESSION", "stderr": "", "stdout": ""
                }],
                "duration": 0.551, "id": None, "name": "com.suite.two.Suite", "stderr": "",
                "stdout": "", "timestamp": "2018-01-23T23:05:17"
            }]}
        events = eg.generate_events(esl)
        nose.tools.ok_(len(events) == 7,
                       "Wrong number of events returned ({}): {}".format(len(events), events))
        results = [
            {'testActions': [], 'age': 0, 'className': 'com.test.one.Test', 'duration': 0.638,
             'errorDetails': None, 'errorStackTrace': None, 'failedSince': 0, 'name': 'testone',
             'skipped': False, 'skippedMessage': None, 'status': 'PASSED', 'stderr': '',
             'stdout': '', 'suite': 'com.suite.one.Suite', 'type': 'case',
             'errorDetailsTruncated': None},
            {'testActions': [], 'age': 0, 'className': 'com.test.two.Test', 'duration': 0.053,
             'errorDetails': None, 'errorStackTrace': None, 'failedSince': 0, 'name': 'testtwo',
             'skipped': False, 'skippedMessage': None, 'status': 'SKIPPED', 'stderr': '',
             'stdout': '', 'suite': 'com.suite.one.Suite', 'type': 'case',
             'errorDetailsTruncated': None},
            {'testActions': [], 'age': 0, 'className': 'com.test.three.Test', 'duration': 0.053,
             'errorDetails': 'Error Detail', 'errorStackTrace': None, 'failedSince': 0,
             'name': 'testthree', 'skipped': False, 'skippedMessage': None, 'status': 'FAILED',
             'stderr': '', 'stdout': '', 'suite': 'com.suite.one.Suite', 'type': 'case',
             'errorDetailsTruncated': 'Error Detail'},
            {'duration': 1516748670, 'id': None, 'name': 'com.suite.one.Suite', 'stderr': '',
             'stdout': '', 'timestamp': '1970-01-01T00:00:00', 'passCount': 1, 'skipCount': 1,
             'failCount': 1, 'unknownCount': 0, 'totalCount': 3, 'type': 'suite'},
            {'testActions': [], 'age': 0, 'className': 'com.test.three.Test', 'duration': 0.551,
             'errorDetails': None, 'errorStackTrace': None, 'failedSince': 0, 'name': 'testfour',
             'skipped': False, 'skippedMessage': None, 'status': 'REGRESSION', 'stderr': '',
             'stdout': '', 'suite': 'com.suite.two.Suite', 'type': 'case',
             'errorDetailsTruncated': None},
            {'duration': 0.551, 'id': None, 'name': 'com.suite.two.Suite', 'stderr': '',
             'stdout': '', 'timestamp': '2018-01-23T23:05:17', 'passCount': 0, 'skipCount': 0,
             'failCount': 0, 'unknownCount': 1, 'totalCount': 1, 'type': 'suite'},
            {'_class': 'hudson.tasks.junit.TestResult', 'testActions': [], 'duration': 4550245900,
             'empty': False, 'failCount': 2, 'passCount': 1, 'skipCount': 1, 'type': 'total',
             'totalCount': 4}]

        for idx, event in enumerate(events):
            nose.tools.ok_(event == results[idx],
                           "Bad event[{}] returned: {}".format(idx, events))



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'mvillene'

from ..interface import EventGenerator
import logging

LOGGER = logging.getLogger(__name__)


class StageEvent(EventGenerator):
    """
    """
    def __init__(self):
        super().__init__()

    def get_fields(self):
        return super().get_fields()

    def generate_events(self, esl):
        """
        return a list of objects to send as events

        :param esl: the calling es_logger object
        :type console_log: object
        :returns: list(dict(str:?))
        """
        LOGGER.debug("Starting: {}".format(type(self).__name__))

        output_list = []

        stages_report = esl.get_stages()

        if stages_report is not None:

            stages = stages_report.pop('stages', None)
            stages_report.pop('name', None)

            for stage in stages:
                output_list.append(stage)

        LOGGER.debug("Finished: {}".format(type(self).__name__))
        return output_list



#!/usr/bin/env python

# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import argparse
import asyncio
import configparser
import es_logger
import json
import logging
import os
import signal
import sys
import urllib
import zmq
from zmq.asyncio import Context


# Configure logging for the daemon
def configure_logging(debug=False):  # pragma: no cover
    if debug:
        log_level = logging.DEBUG
    else:
        log_level = logging.INFO

    logging.basicConfig(level=log_level,
                        format='%(asctime)s %(name)s %(levelname)s %(message)s')


# Base Exception Class
class ZMQClientError(Exception):
    pass


# Misconfiguration exception class
class ZMQClientMisconfiguration(ZMQClientError):
    pass


# Class for running es-logger as Jenkins ZMQ listener
class ESLoggerZMQDaemon(object):
    def __init__(self, args):
        self.listener = None
        self.tasks = []
        self.env_vars = ['JENKINS_URL', 'JENKINS_USER', 'JENKINS_PASSWORD', 'PROCESS_CONSOLE_LOGS',
                         'GATHER_BUILD_DATA', 'GENERATE_EVENTS', 'LOGSTASH_SERVER', 'LS_USER',
                         'LS_PASSWORD']
        self.process_console_logs = ''
        self.gather_build_data = ''
        self.generate_events = ''
        self.async_main_sleep = 30
        self.worker_sleep = 15
        self.test_zmq = args.test_zmq

    # Read the configuration
    def configure(self, config_file='es-logger.ini'):
        config = configparser.ConfigParser()
        config.read(config_file)

        if 'zmq' in config:
            self.num_workers = int(config['zmq'].get('num_workers', 3))
            self.zmq_publisher = config['zmq'].get('zmq_publisher')

        if 'jenkins' in config:
            self.jenkins_url = config['jenkins'].get('jenkins_url')
            self.jenkins_user = config['jenkins'].get('jenkins_user')
            self.jenkins_password = config['jenkins'].get('jenkins_password')

        if 'logstash' in config:
            self.logstash_server = config['logstash'].get('logstash_server')
            self.ls_user = config['logstash'].get('ls_user')
            self.ls_password = config['logstash'].get('ls_password')

        # Set defaults plugins to all unless overridden
        if 'plugins' not in config:
            config['plugins'] = {}
        self.plugins = {}

        console_log_processor = ' '.join(
            es_logger.EsLogger.list_plugins(True, ['console_log_processor']))
        self.process_console_logs = config['plugins'].get('process_console_logs',
                                                          console_log_processor)
        logging.info("Using console_log_processor plugins: {}".format(self.process_console_logs))
        self.get_plugin_config("process_console_logs", self.process_console_logs.split(" "), config)

        gather_build_data = ' '.join(
            es_logger.EsLogger.list_plugins(True, ['gather_build_data']))
        self.gather_build_data = config['plugins'].get('gather_build_data', gather_build_data)
        logging.info("Using gather_build_data plugins: {}".format(self.gather_build_data))
        self.get_plugin_config("gather_build_data", self.gather_build_data.split(" "), config)

        event_generator = ' '.join(es_logger.EsLogger.list_plugins(True, ['event_generator']))
        self.generate_events = config['plugins'].get('generate_events', event_generator)
        logging.info("Using generate_events plugins: {}".format(self.generate_events))
        self.get_plugin_config("generate_events", self.generate_events.split(" "), config)

        self.validate_config()

        # Set the necessary environment variables
        for var in self.env_vars:
            self.set_in_env(var, getattr(self, var.lower()))

        # Iterate over the plugin configuration and set it in the environment
        for plugin in self.plugins.keys():
            for var in self.plugins[plugin].keys():
                self.set_in_env(var, self.plugins[plugin][var])

    # lower case var to upper case env var
    @staticmethod
    def set_in_env(var, val):
        var_name = var.upper()
        if val is not None and val != '':
            os.environ[var_name] = val
            logging.debug('Setting env var {}'.format(var_name))

    # Check for plugin-specific configuration
    def get_plugin_config(self, plugin_type, plugin_list, config):
        for plugin in plugin_list:
            plugin_key = plugin_type + ":" + plugin
            if plugin_key in config:
                self.plugins[plugin_key] = {}
                for key in config[plugin_key]:
                    self.plugins[plugin_key][key] = config[plugin_key].get(key)

    # Make sure we are correctly configured
    def validate_config(self):
        # Confirm configuration or raise an exception
        unset_attr = []
        for attr in ['jenkins_url', 'jenkins_user', 'jenkins_password',
                     'logstash_server', 'zmq_publisher']:
            if (not hasattr(self, attr)) or getattr(self, attr) is None:
                unset_attr.append(attr)
        if len(unset_attr) > 0:
            e = ZMQClientMisconfiguration(
                "Unconfigured for daemon operation, check config "
                "for the following variables:\n{}".format(unset_attr))
            raise(e)

    # Strip off trailing slashes, and remove each "job" path element
    @staticmethod
    def get_project_name(job):
        project = urllib.parse.urlsplit(job).path
        # Ensure no trailing slash
        project = project.rstrip('/')
        # Ensure no job elements in path, as python-jenkins expects to add them
        project = '/'.join([p for p in project.split('/') if p != 'job'])
        # Finally, unquote the string, as python-jenkins expects to url encode
        project = urllib.parse.unquote(project)
        return project

    # Processing function to validate ZMQ connection
    def test_zmq_task(self, msg):
        var = json.loads(''.join(msg[0].decode("utf-8").split()[1:]))
        job = self.get_project_name(var['url'])
        number = var['build'].get('number')
        phase = var['build'].get('phase')
        logging.info(f'Got event: job [{job}] number [{number}] phase [{phase}]')
        return 0

    # Processing function to run es-logger
    def es_logger_task(self, msg):
        var = json.loads(''.join(msg[0].decode("utf-8").split()[1:]))
        phase = var['build'].get('phase')
        if phase == 'FINISHED':
            job = self.get_project_name(var['url'])
            number = var['build'].get('number')
            logging.info("Process {} number {} on {}".format(job, number, self.jenkins_url))
            # Create and configure the ES-Logger instance
            esl = es_logger.EsLogger(console_length=32500, targets=['logstash'])
            esl.es_job_name = job
            esl.es_build_number = int(number)
            esl.gather_all()
            status = esl.post_all()
            logging.info("{} number {} status {}".format(job, number, status))
        else:
            logging.debug('Not collecting from job in phase {}'.format(phase))
            status = None
        return status

    # Worker to pull tasks off queue, process message, and execute
    async def worker(self, name, process_func):
        logging.info("{} Starting".format(name))
        current_task = asyncio.current_task()
        while not current_task.done():
            try:
                logging.debug("{} waiting for work".format(name))
                msg = await asyncio.wait_for(self.queue.get(), self.worker_sleep)
                logging.debug("{} processing msg {}".format(name, msg))
                result = getattr(self, process_func)(msg)
                self.queue.task_done()
                logging.debug("{} result {}".format(name, result))
            except asyncio.TimeoutError:
                logging.debug("{} timeout waiting for work, looping".format(name))
            except asyncio.CancelledError:
                logging.info("{} cancelled, finishing".format(name))
                break
        logging.info("{} Finished".format(name))
        return 0

    # Drain the queue to enable clean shutdown
    async def drain_queue(self):
        status_list = []
        qsize = self.queue.qsize()
        processed = 0
        logging.info("Draining queue of size {}".format(qsize))
        while self.queue.qsize() != 0:
            msg = self.queue.get_nowait()
            logging.debug("Processing msg {}".format(msg))
            result = self.es_logger_task(msg)
            self.queue.task_done()
            logging.debug("Result {}".format(result))
            if result is not None:
                status_list.append(result)
            processed += 1
        logging.info("Queue drained, processed {}".format(processed))
        return status_list

    # Connection function executed as listener task
    async def recv(self):
        logging.info('Listener Starting against {}'.format(self.zmq_publisher))
        # Connect to ZMQ
        ctx = Context.instance()
        s = ctx.socket(zmq.SUB)
        s.connect(self.zmq_publisher)
        s.subscribe(b'')

        current_task = asyncio.current_task()
        while not current_task.done():
            try:
                logging.debug("Listener waiting for message")
                msg = await asyncio.wait_for(s.recv_multipart(), None)
                logging.debug("Adding {} to queue {}".format(msg, self.queue.qsize()))
                await self.queue.put(msg)
            except asyncio.CancelledError:
                logging.info("Listener cancelled, finishing")
                break
        s.close()
        logging.info("Listener Finished")
        return 0

    # Start the threads for processing
    def start(self):
        logging.info('Starting')
        # Setup signal handlers
        self.loop = asyncio.get_running_loop()
        for signame in ('SIGINT', 'SIGTERM'):
            self.loop.add_signal_handler(getattr(signal, signame), self.stop)
        self.queue = asyncio.Queue()
        # Create an asynchronous listener task
        self.listener = asyncio.create_task(self.recv())
        # Create worker tasks to process the queue concurrently.
        self.tasks = []
        if self.test_zmq:
            process_func = 'test_zmq_task'
        else:
            process_func = 'es_logger_task'
        for i in range(self.num_workers):
            task = asyncio.create_task(self.worker(f'worker-{i}', process_func))
            self.tasks.append(task)
        logging.info(f'Started {self.num_workers} workers using {process_func}')

    # Cancel the threads for stopping
    def stop(self):
        if self.listener is not None:
            logging.info("Stopping listener")
            self.listener.cancel()
        if len(self.tasks) > 0:
            logging.info("Stopping tasks")
            for task in self.tasks:
                task.cancel()
        all_tasks = asyncio.Task.all_tasks()
        logging.debug("All Tasks: {}".format(all_tasks))
        logging.info("Stopped, waiting for tasks to finish")

    # Check a task is in a good state
    def check_task(self, task):
        if task.done():
            return False
        return True

    # Check that the listener is still running correctly
    def check_listener(self):
        logging.debug(self.listener)
        return self.check_task(self.listener)

    # Check that worker tasks are still running
    def check_tasks(self):
        logging.debug(self.tasks)
        for task in self.tasks:
            if not self.check_task(task):
                return False
        return True

    # Return a list of all tasks started
    def all_tasks(self):
        return [self.listener] + self.tasks

    # Boolean declaring whether all tasks are done or not
    def tasks_done(self):
        for task in self.all_tasks():
            if not task.done():
                return False
        return True

    # The ASync executed main function to control lifecycle
    async def async_main(self):
        # Start, monitor, then stop
        self.start()
        # Give some time to start up (also helps tests run faster)
        await asyncio.sleep(2)
        logging.info("Started tasks, entering status check loop")
        while not self.tasks_done():
            if not self.check_listener():
                logging.warning("Listener not running")
                self.stop()
                break
            if not self.check_tasks():
                logging.warning("Tasks not running")
                self.stop()
                break
            await asyncio.sleep(self.async_main_sleep)

        logging.info("Exited status check loop")

        # Ensure all the tasks get completed
        drain_status = await self.drain_queue()

        # Wait for all of the coroutines to finish and gather the result
        logging.info("Gathering task statuses")
        status_list = await asyncio.gather(*([self.listener] + self.tasks), return_exceptions=True)
        status_list = status_list + drain_status
        status = 0
        for s in status_list:
            if isinstance(s, Exception):
                logging.warning("Exception: {}".format(s))
                status += 1
            elif isinstance(s, int):
                status += s
            else:
                logging.warning("Unknown status: {}".format(s))
                status += 1
        return status

    # Synchronous main to trigger async main only,
    # after loading configuration and setting environment
    def main(self):
        self.configure()
        status = asyncio.run(self.async_main())
        logging.info("Finished")
        return status


# Argument parsing shouldn't need coverage, just effects of flags elsewhere
def parse_args():  # pragma: no cover
    desc = '''
Run a ZMQ subscription to connect to the Jenkins Event Publisher (via ZMQ PUB SUB) plugin.
https://plugins.jenkins.io/zmq-event-publisher/
'''
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=desc)
    parser.add_argument('-t', '--test-zmq', action='store_true',
                        help='Print details instead of processing with es-logger.')
    parser.add_argument('-d', '--debug', action='store_true',
                        help='Print debug logs to console during execution')
    args = parser.parse_args()
    return args


# Shouldn't need individual unit testing, keep as simple as possible
def main():  # pragma: no cover
    args = parse_args()
    configure_logging(args.debug)
    d = ESLoggerZMQDaemon(args)
    status = d.main()
    sys.exit(status)



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import itertools
import json
import logging
import urllib
from ..interface import ConsoleLogEventRegex, EventGenerator
import re

LOGGER = logging.getLogger(__name__)


class AnsibleRecapEvent(EventGenerator):
    """
    """
    def __init__(self):
        super().__init__()

    def get_fields(self):
        return super().get_fields()

    def generate_events(self, esl):
        """
        Parse the console log and return a list for ES

        :param esl: The es_logger object being collected
        :returns: list(obj)
        """
        LOGGER.debug("Starting: {}".format(type(self).__name__))
        output_list = []
        # Find the play, e.g.: PLAY [Play name or hosts] **...**
        # Find the recap start, e.g.: PLAY RECAP **...**
        # Find the recap end, e.g.: Total --...-- 1861.60s (31 min 2 sec)
        recap_regex = re.compile(
                r'PLAY\s*\[(?P<play>.*)\]\s*\**\s*\n' +    # Group 1 matches the play name
                r'(?:.*\n(?!.*PLAY RECAP))*?' +            # Discard lines to 1st recap (non-greedy)
                r'\s*PLAY RECAP \**\s*\n' +                # Match the recap start
                r'(?P<hosts>(?:[^\s].*\n)*)' +             # Match all host lines
                r'(?:\s*\n)*' +                            # Blank lines
                r'(?P<tasks>(?:TASK:\s*.*\n)*)' +          # These are individual timings
                r'(?:\s*\n)*' +                            # Blank lines
                r'Total\s*[-]*\s*(?P<total>[^\s]*)s.*\n',  # Total time
                re.MULTILINE)
        # Find the play, e.g.: PLAY [Play name or hosts] **...**
        # Find the recap start, e.g.: PLAY RECAP **...**
        # Find the recap end, which is finished with all tasks recapped
        profile_regex = re.compile(
                r'PLAY\s*\[(?P<play>.*)\]\s*\**\s*\n' +            # Group 1 matches the play name
                r'(?:.*\n(?!.*PLAY RECAP))*?' +                    # Discard lines to 1st recap
                r'\s*PLAY RECAP \**\s*\n' +                        # Match the recap start
                r'(?P<hosts>(?:.*\n(?!.*PLAY RECAP))*?)' +         # Match all host lines,
                                                                   #    and not a recap
                r'(?:\s*\n)+' +                                    # Blank lines
                r'.*?\s+(?P<total>[^\s]+)\s+\*+\s*\n' +            # Match total time
                r'\s*={79}\s*\n' +                                 # Separator
                r'(?P<tasks>(?:(?:.*\s:\s)?.*\s[0-9.]+s\s*\n)*)',  # These are individual timings
                re.MULTILINE)
        # Task output matcher, get a description and a timing from, e.g.:
        #  ssh-tunnel : Create the ssh-tunnel -...- 6.21s (not verified)
        task_regex = re.compile(r'\s*(?P<task>.*?)\s+[-]*\s+(?P<time>[0-9.]+)s.*?\n', re.MULTILINE)
        host_regex = re.compile(r'\s*(?P<host>[^\s]*)\s*:\s*(?P<status>.*?)\s*\n', re.MULTILINE)

        # Using coloured output can lead to ANSI escapes buried in the string
        ansi_escape = re.compile(r'\x1B[@-_][0-?]*[ -/]*[@-~]')
        console_no_ansi = ansi_escape.sub('', esl.console_log)

        for recap_match in itertools.chain(recap_regex.finditer(console_no_ansi),
                                           profile_regex.finditer(console_no_ansi)):
            play = recap_match.group('play')
            if ':' in recap_match.group('total'):
                parts = recap_match.group('total').split(':')
                total_time = float(int(parts[-3]) * 60 * 60 +
                                   int(parts[-2]) * 60 +
                                   float(parts[-1]))
            else:
                total_time = float(recap_match.group('total'))
            # Process individual hosts
            for host_match in host_regex.finditer(recap_match.group('hosts')):
                add_event = {}
                add_event['play'] = play
                add_event['host'] = host_match.group('host')
                for status in host_match.group('status').split():
                    try:
                        string, count = status.split('=')
                        add_event[string] = int(count)
                    except ValueError as e:
                        LOGGER.error("Bad status match in hosts:\n{}".format(
                                     recap_match.group('hosts')))
                        raise(e)
                output_list.append(add_event)
            # Process individual tasks
            for task_match in task_regex.finditer(recap_match.group('tasks')):
                time = float(task_match.group('time'))
                add_event = {}
                add_event['play'] = play
                # Add the total play time
                add_event['total'] = total_time
                add_event['time_percentage'] = (time / total_time) * 100
                add_event['description'] = task_match.group('task')
                add_event['time'] = time
                output_list.append(add_event)
        LOGGER.debug("Finished: {}".format(type(self).__name__))
        # Return the data
        return output_list


class AnsibleFatalGenerator(EventGenerator):
    """
    Process the ansible play output for any lines beginning "fatal"
    """

    def get_fields(self):
        return super(AnsibleFatalGenerator, self).get_fields()

    def generate_events(self, esl):
        """
        Parse the console log and return a list for ES

        :param esl: The es_logger object being collected
        :returns: list(obj)
        """
        LOGGER.debug("Starting: {}".format(type(self).__name__))
        output_list = []
        # Log ansible fatal errors: e.g. fatal: [<host>]: FAILED! => <json_status>
        fatal_regex = re.compile(
                r'^fatal:\s*\[(?P<hostname>\S+)\]:\s*' +    # Group 1 matches the hostname
                r'(?P<type>FAILED|UNREACHABLE)!\s*=>\s*' +  # precursor to json status element
                r'(?P<error>{.*?})\s*$',                    # json status to parse in to end of line
                re.MULTILINE | re.DOTALL)
        for fatal_match in fatal_regex.finditer(esl.console_log):
            add_event = {}
            add_event['hostname'] = fatal_match.group('hostname')
            add_event['type'] = fatal_match.group('type')
            try:
                add_event['data'] = json.loads(urllib.parse.quote(fatal_match.group('error'),
                                                                  '":{} ,'))
            except json.decoder.JSONDecodeError:
                add_event['bad_data'] = fatal_match.group('error')
            output_list.append(add_event)
        LOGGER.debug("Finished: {}".format(type(self).__name__))
        # Return the data
        return output_list


class AnsibleConsoleLogRegex(ConsoleLogEventRegex):
    """
    """
    def get_regex(regex_list):
        """
        Ansible warnings and deprecations
        """
        regex_list.append(("ansible warning", re.compile(
            r"^\[WARNING\]:\s*(?P<warning_text>(?:.*?\n)+?)(?=^\[.*|\s*\n)",
            re.MULTILINE)))

        regex_list.append(("ansible deprecation", re.compile(
            r"^\[DEPRECATION WARNING\]:\s*(?P<deprecation_text>(?:.*?\n)+?)(?=^\[.*|\s*\n)",
            re.MULTILINE)))

        return regex_list



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

from ..interface import EventTarget
import logging
import os
import requests
import time
import urllib3.exceptions

LOGGER = logging.getLogger(__name__)


class LogstashPostError(Exception):
    pass


class LogstashTarget(EventTarget):
    """
    """
    def __init__(self):
        super().__init__()

        self.logstash_server = None
        self.ls_user = None
        self.ls_password = None
        self.ls_session = None

        self.timeout_sleep = 2

        # Where to send the json event
        self.logstash_server = self.get_logstash_server()
        self.ls_user = self.get_ls_user()
        self.ls_password = self.get_ls_password()

    @staticmethod
    def get_help_string():
        return '''
Logstash Target Environment Variables:
    LOGSTASH_SERVER         The server to send events to
    LS_USER                 The user for logstash access
    LS_PASSWORD             The password for logstash access
'''

    def validate(self):
        return True

    # Post to ES
    def send_event(self, json_event):
        # We see a lot of logstash timeout errors
        post_attempts = []
        r = None
        # If we successfully post, r becomes the response
        while r is None:
            try:
                session = self.get_session()
                r = session.post(self.logstash_server, json=json_event)
            except (requests.exceptions.ReadTimeout, urllib3.exceptions.ProtocolError) as exc:
                post_attempts.append(exc)
                LOGGER.warn("Setting session to None on post_attempt {}".format(
                            len(post_attempts)))
                self.ls_session = None
                if len(post_attempts) >= 5:
                    raise LogstashPostError(
                        "Logstash post errors {}".format(post_attempts)) from exc
                time.sleep(self.timeout_sleep)
            except Exception as exc:
                raise LogstashPostError(
                    "Logstash post error on attempt {}".format(post_attempts)) from exc
        LOGGER.debug("Posted event, result {}".format(r.ok))
        if r.ok:
            return 0
        return 1

    def get_logstash_server(self):
        if not self.logstash_server:
            return os.environ.get('LOGSTASH_SERVER')
        return self.logstash_server

    def get_ls_user(self):
        if not self.ls_user:
            return os.environ.get('LS_USER')
        return self.ls_user

    def get_ls_password(self):
        if not self.ls_password:
            return os.environ.get('LS_PASSWORD')
        return self.ls_password

    def get_session(self):
        if self.ls_session is None:
            LOGGER.debug("Creating session against {}".format(self.logstash_server))
            self.ls_session = requests.Session()
            self.ls_session.auth = (self.ls_user, self.ls_password)
        return self.ls_session



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import es_logger
import nose
import sys
import unittest.mock

DEFAULT_CONSOLE_LENGTH = 32500


class ExitException(Exception):
    pass


class TestCli(object):

    # Test call flow with good options
    @unittest.mock.patch('es_logger.EsLogger', autospec=True)
    def test_cli(self, mock_esl):
        sys.argv = ['es-logger']
        mock_event = unittest.mock.MagicMock()
        mock_es_info = unittest.mock.MagicMock()
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).es_info = mock_es_info
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).post.return_value = 0
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).finish.return_value = 0
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).events = [mock_event,
                                                                              mock_event]
        with unittest.mock.patch('sys.exit', side_effect=ExitException) as mock_exit:
            nose.tools.assert_raises(ExitException, es_logger.cli.main)
            mock_exit.assert_called_with(0)
            calls = [unittest.mock.call.get_event_target_plugin_help(),
                     unittest.mock.call().gather_all(),
                     unittest.mock.call().dump(mock_es_info),
                     unittest.mock.call().post(mock_es_info),
                     unittest.mock.call().dump(mock_event),
                     unittest.mock.call().post(mock_event),
                     unittest.mock.call().dump(mock_event),
                     unittest.mock.call().post(mock_event),
                     unittest.mock.call().finish()]
            print(mock_esl.method_calls)
            nose.tools.ok_(mock_esl.method_calls == calls)

    # Test mutually exclusive options
    @unittest.mock.patch('es_logger.EsLogger', autospec=True)
    def test_exclusive_options(self, mock_esl):
        sys.argv = ['es-logger', '--no-dump', '--no-post']
        with unittest.mock.patch('sys.exit', side_effect=ExitException()) as mock_exit:
            nose.tools.assert_raises(ExitException, es_logger.cli.main)
            mock_exit.assert_called_with(2)

    # Test console length
    @unittest.mock.patch('es_logger.EsLogger', autospec=True)
    def test_console_length(self, mock_esl):
        mock_esl(0, []).events = []
        mock_esl(0, []).finish.return_value = 0
        sys.argv = ['es-logger', '-c', '1000', '-e']
        with unittest.mock.patch('sys.exit', side_effect=ExitException()) as mock_exit:
            nose.tools.assert_raises(ExitException, es_logger.cli.main)
            mock_exit.assert_called_with(0)
        print(mock_esl.mock_calls)
        mock_esl.assert_called_with(console_length=1000, targets=['logstash'])

    # Test event only generation
    @unittest.mock.patch('es_logger.EsLogger', autospec=True)
    def test_event_only(self, mock_esl):
        sys.argv = ['es-logger', '-e']
        mock_event = unittest.mock.MagicMock()
        mock_es_info = unittest.mock.MagicMock()
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).es_info = mock_es_info
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).post.return_value = 0
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).finish.return_value = 0
        mock_esl(console_length=DEFAULT_CONSOLE_LENGTH, targets=[]).events = [mock_event,
                                                                              mock_event]
        with unittest.mock.patch('sys.exit', side_effect=ExitException) as mock_exit:
            nose.tools.assert_raises(ExitException, es_logger.cli.main)
            mock_exit.assert_called_with(0)
            calls = [unittest.mock.call.get_event_target_plugin_help(),
                     unittest.mock.call().gather_all(),
                     unittest.mock.call().dump(mock_event),
                     unittest.mock.call().post(mock_event),
                     unittest.mock.call().dump(mock_event),
                     unittest.mock.call().post(mock_event),
                     unittest.mock.call().finish()]
            print(mock_esl.method_calls)
            nose.tools.ok_(mock_esl.method_calls == calls, "Calls made don't match expected")

    # Test list plugins
    def test_list_plugins(self):
        sys.argv = ['es-logger', '-p']
        with unittest.mock.patch('sys.exit', side_effect=ExitException()) as mock_exit:
            nose.tools.assert_raises(ExitException, es_logger.cli.main)
            mock_exit.assert_called_with(0)

    def test_cli_debug(self):
        sys.argv = ['es-logger', '--debug', '-p']
        with unittest.mock.patch.object(sys, 'exit', side_effect=ExitException) as mock_exit:
            with unittest.mock.patch('logging.basicConfig') as mock_logging:
                nose.tools.assert_raises(ExitException, es_logger.cli.main)
                mock_exit.assert_called_with(0)
                mock_logging.assert_called_with(
                    format='%(asctime)s %(name)s %(levelname)s %(message)s', level=10)



__author__ = 'jonpsull'

import es_logger
import es_logger.plugins.console_log_events
from .event_generator_base import TestEventGenerator
import nose.tools
import unittest.mock


class TestConsoleLog(TestEventGenerator):
    def setup(self):
        self.esl = unittest.mock.MagicMock(
            spec=es_logger.EsLogger, console_log='Log', build_info={}, es_info={},
            data_name='eslogger')

    def test_read_regex_file(self):
        with unittest.mock.patch('os.path.isfile') as mock_isfile, \
                unittest.mock.patch('builtins.open') as mock_file, \
                unittest.mock.patch(
                    'es_logger.plugins.console_log_events.json.loads') as mock_json_loads:
            mock_isfile.return_value = True
            mock_json_loads.return_value = [{"name": "one", "pattern": "one (?P<one>.*)$"},
                                            {"name": "two", "pattern": "two(?P<two>.*)$"}]
            p = es_logger.plugins.console_log_events.ConsoleLogEvent()
            self.esl.console_log = ''
            self.esl.console_log += '''
one match
two matches
'''
            ret = p.generate_events(self.esl)
            self.return_length_check(ret, 2)
            mock_file_calls = [unittest.mock.call('console_log_event_regex.json', 'r'),
                               unittest.mock.call().__enter__(),
                               unittest.mock.call().__exit__(None, None, None)]
            nose.tools.ok_(mock_file.mock_calls == mock_file_calls,
                           "Wrong file calls:\n{}".format(mock_file.mock_calls))

        one = [x for x in ret if x['name'] == 'one']
        self.return_length_check(one, 1)
        self.check_named_match(one[0], 'one', 'match')

        two = [x for x in ret if x['name'] == 'two']
        self.return_length_check(two, 1)
        self.check_named_match(two[0], 'two', ' matches')

    def test_ConsoleLogEvent(self):
        p = es_logger.plugins.console_log_events.ConsoleLogEvent()
        self.expected_fields_check(p.get_fields(), ['NODE_NAME', 'NODE_LABELS'])
        self.esl.console_log = ''

        self.esl.console_log += '''
Running on jenkins-agent-1 in /mnt/jenkins/workspace/job-name-random
'''
        self.esl.console_log += '\n'
        self.esl.console_log += '''
Error response from daemon: The error message
next line

preceeding line
same line unknown blob trailing
'''
        self.esl.console_log += '\n'
        self.esl.console_log += 'Error response from daemon: The error message net/http: request'
        self.esl.console_log += ' canceled (Client.Timeout exceeded while awaiting headers)'
        self.esl.console_log += '''
Error response from daemon: missing signature key

Pulling repository docker_repository
Error: image docker_image not found

Error: image docker_image not found
'''
        self.esl.console_log += '\n'
        self.esl.console_log += 'Error pulling image (docker_image:tag) from docker_repository,'
        self.esl.console_log += ' endpoint: docker_endpoint, The error message'
        self.esl.console_log += '''
Please login prior to pull:
Error: Cannot perform an interactive login from a non TTY device

received unexpected HTTP status: http_status
'''
        self.esl.console_log += '\n'
        self.esl.console_log += 'devmapper: Thin Pool has 100000 free data blocks which is less'
        self.esl.console_log += ' than minimum required 100000 free data blocks. Create more free'
        self.esl.console_log += ' space in thin pool or use dm.min_free_space option to change'
        self.esl.console_log += ' behavior'
        self.esl.console_log += '''
Total reclaimed space 4MB

/dev/stdout: resource temporarily unavailable

java.class.AnException: This is the exception

ERROR: Error message
next line

FATAL: Fatal message
next line

remote file operation failed: remote failed text
next line
'''

        self.esl.console_log += '''
[DEPRECATION WARNING]: Multi
line
deprecation
[WARNING]: One line warning
[WARNING]: Multi
line
warning

'''
        ret = p.generate_events(self.esl)
        self.return_length_check(ret, 23)

        jenkins_agent = [x for x in ret if x['name'] == 'jenkins agent']
        self.return_length_check(jenkins_agent, 1)
        self.check_named_match(jenkins_agent[0], 'jenkins_agent', 'jenkins-agent-1')
        self.check_named_match(jenkins_agent[0], 'workspace',
                               '/mnt/jenkins/workspace/job-name-random')

        docker_error = [x for x in ret if x['name'] == 'docker error']
        self.return_length_check(docker_error, 3)
        self.check_named_match(docker_error[0], 'docker_error', ' The error message')

        blob_error = [x for x in ret if x['name'] == 'docker blob error']
        self.return_length_check(blob_error, 1)
        self.check_named_match(blob_error[0], 'preceding', 'preceeding line')
        self.check_named_match(blob_error[0], 'blob', 'same line unknown blob trailing')

        docker_timeout = [x for x in ret if x['name'] == 'docker timeout']
        self.return_length_check(docker_timeout, 1)
        self.check_named_match(docker_timeout[0], 'err', ' The error message ')

        missing_sig = [x for x in ret if x['name'] == 'docker missing signature']
        self.return_length_check(missing_sig, 1)

        image_not_found = [x for x in ret if x['name'] == 'docker image not found']
        self.return_length_check(image_not_found, 3)
        self.check_named_match(image_not_found[0], 'docker_repository', 'docker_repository')
        self.check_named_match(image_not_found[0], 'docker_image', 'docker_image')
        self.check_named_match(image_not_found[1], 'image', 'docker_image')
        self.check_named_match(image_not_found[2], 'image', 'docker_image')

        pull_error = [x for x in ret if x['name'] == 'docker pull error']
        self.return_length_check(pull_error, 2)
        self.check_named_match(pull_error[0], 'docker_tag', 'docker_image:tag')
        self.check_named_match(pull_error[0], 'docker_registry', 'docker_repository')
        self.check_named_match(pull_error[0], 'docker_endpoint', 'docker_endpoint')
        self.check_named_match(pull_error[0], 'error', 'The error message')

        http_status = [x for x in ret if x['name'] == 'http_status']
        self.return_length_check(http_status, 1)
        self.check_named_match(http_status[0], 'http_status', ' http_status')

        thin_pool_full = [x for x in ret if x['name'] == 'thin pool full']
        self.return_length_check(thin_pool_full, 1)

        reclaimed_space = [x for x in ret if x['name'] == 'reclaimed space']
        self.return_length_check(reclaimed_space, 1)

        stdout_unavail = [x for x in ret if x['name'] == 'stdout temporarily unavailable']
        self.return_length_check(stdout_unavail, 1)

        java_exception = [x for x in ret if x['name'] == 'java exception']
        self.return_length_check(java_exception, 1)
        self.check_named_match(java_exception[0], 'exception', 'java.class.AnException')
        self.check_named_match(java_exception[0], 'exception_text', 'This is the exception')

        jenkins_error = [x for x in ret if x['name'] == 'jenkins error']
        self.return_length_check(jenkins_error, 1)
        self.check_named_match(jenkins_error[0], 'error_text', 'Error message')

        jenkins_fatal = [x for x in ret if x['name'] == 'jenkins fatal error']
        self.return_length_check(jenkins_fatal, 1)
        self.check_named_match(jenkins_fatal[0], 'fatal_text', 'Fatal message')

        jenkins_remote_failed = [x for x in ret if x['name'] == 'remote file operation failed']
        self.return_length_check(jenkins_remote_failed, 1)
        self.check_named_match(jenkins_remote_failed[0], 'remote_fail_text', 'remote failed text')

        ansible_warning = [x for x in ret if x['name'] == 'ansible warning']
        self.return_length_check(ansible_warning, 2)
        self.check_named_match(ansible_warning[0], 'warning_text', 'One line warning\n')
        self.check_named_match(ansible_warning[1], 'warning_text', 'Multi\nline\nwarning\n')

        ansible_deprecation = [x for x in ret if x['name'] == 'ansible deprecation']
        self.return_length_check(ansible_deprecation, 1)
        self.check_named_match(ansible_deprecation[0], 'deprecation_text',
                               'Multi\nline\ndeprecation\n')



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import asyncio
import es_logger
import nose
import os
import unittest.mock


class TestZMQClient(object):
    config_dict = {}
    sample_completed_message = '''onFinalized {"name":"sample-job",
        "url":"job/folder/job/sample-job/","build":{
        "full_url":"https://joc.example.com/jenkins/jenkins_url/job/folder/job/sample-job/123/",
        "number":123,"phase":"COMPLETED","status":"SUCCESS","url":"job/folder/job/sample-job/123/",
        "node_name":"","node_description":"the master Jenkins node",
        "host_name":"jenkins_url"}}'''.encode('ascii')
    sample_completed_message_log = 'b\'onFinalized {"name":"sample-job",\\n        ' +\
        '"url":"job/folder/job/sample-job/","build":{\\n        "full_url":' +\
        '"https://joc.example.com/jenkins/jenkins_url/job/folder/job/sample-job/123/",\\n' +\
        '        "number":123,"phase":"COMPLETED","status":"SUCCESS","url":' +\
        '"job/folder/job/sample-job/123/",\\n        "node_name":"","node_description":' +\
        '"the master Jenkins node",\\n        "host_name":"jenkins_url"}}\''
    sample_finished_message = '''onFinalized {"name":"sample-job",
        "url":"job/folder/job/sample-job/","build":{
        "full_url":"https://joc.example.com/jenkins/jenkins_url/job/folder/job/sample-job/123/",
        "number":123,"phase":"FINISHED","status":"SUCCESS","url":"job/folder/job/sample-job/123/",
        "node_name":"","node_description":"the master Jenkins node",
        "host_name":"jenkins_url"}}'''.encode('ascii')
    sample_finished_message_log = 'b\'onFinalized {"name":"sample-job",\\n        ' +\
        '"url":"job/folder/job/sample-job/","build":{\\n        "full_url":' +\
        '"https://joc.example.com/jenkins/jenkins_url/job/folder/job/sample-job/123/",\\n' +\
        '        "number":123,"phase":"FINISHED","status":"SUCCESS","url":' +\
        '"job/folder/job/sample-job/123/",\\n        "node_name":"","node_description":' +\
        '"the master Jenkins node",\\n        "host_name":"jenkins_url"}}\''

    def setup(self):
        self.config_dict = {}
        args = unittest.mock.MagicMock()
        args.debug = False
        args.test_zmq = True
        self.zmqd = es_logger.zmq_client.ESLoggerZMQDaemon(args)
        self.zmqd.async_main_sleep = 5
        self.zmqd.worker_sleep = 5
        self.loop = asyncio.new_event_loop()
        self.loop.set_debug(True)
        asyncio.set_event_loop(self.loop)

    def get_config_item(self, name):
        return self.config_dict[name]

    def set_config_item(self, name, val):
        self.config_dict[name] = val

    def config_contains(self, name):
        return name in self.config_dict

    def config_setup(self, mock_config_parser):
        config = mock_config_parser.return_value
        config.__getitem__.side_effect = self.get_config_item
        config.__setitem__.side_effect = self.set_config_item
        config.__contains__.side_effect = self.config_contains
        return config

    def set_default_config(self, config):
        config['jenkins'] = {}
        config['jenkins']['jenkins_url'] = 'https://jenkins.example.com'
        config['jenkins']['jenkins_user'] = 'jenkins_user'
        config['jenkins']['jenkins_password'] = 'jenkins_password'
        config['logstash'] = {}
        config['logstash']['logstash_server'] = 'https://logstash.example.com:8080'
        config['logstash']['ls_user'] = 'ls_user'
        config['logstash']['ls_password'] = 'ls_password'
        config['zmq'] = {}
        config['zmq']['zmq_publisher'] = 'tcp://jenkins.example.com:8888'

    def set_plugin_config(self, config):
        config['plugins'] = {}
        config['plugins']['process_console_logs'] = 'process_console_logs'
        config['plugins']['gather_build_data'] = 'gather_build_data'
        config['plugins']['generate_events'] = 'generate_events'

    def set_generate_events_plugin_config(self, config):
        config['plugins']['generate_events'] = 'plugin-a pluginb'
        config['generate_events:plugin-a'] = {}
        config['generate_events:pluginb'] = {}
        config['generate_events:plugin-a']["test-plugin-a_env1"] = "test-plugin-a_value1"
        config['generate_events:plugin-a']["test-plugin-a_env2"] = "test-plugin-a_value2"
        config['generate_events:pluginb']["test-pluginb_env1"] = "test-pluginb_value1"

    def set_gather_build_data_plugin_config(self, config):
        config['plugins']['gather_build_data'] = 'plugin'
        config['gather_build_data:plugin'] = {}
        config['gather_build_data:plugin']["test-plugin_env1"] = "test-plugin_value1"
        config['gather_build_data:plugin']["test-plugin_env2"] = "test-plugin_value2"

    def set_process_console_logs_plugin_config(self, config):
        config['plugins']['process_console_logs'] = 'pluginA'
        config['process_console_logs:pluginA'] = {}
        config['process_console_logs:pluginA']["test-pluginA_env1"] = "test-pluginA_value1"
        config['process_console_logs:pluginA']["test-pluginA_env2"] = "test-pluginA_value2"

    async def dummyTask(self, name, sleep=0, return_status=0, exception=None):
        print(f"I am dummyTask {name}")
        # Give 5 seconds to ensure the main loop drops into check status
        if sleep > 0:
            await asyncio.sleep(sleep)
        if exception is not None:
            raise exception
        print(f"dummyTask {name} finishing")
        return return_status

    def test_zmq_client_configure_num_workers(self):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            config['zmq']['num_workers'] = '5'
            self.zmqd.configure()
            nose.tools.ok_(self.zmqd.num_workers == 5,
                           "{} did not match {} ({})".format(5, self.zmqd.num_workers,
                                                             type(self.zmqd.num_workers)))

    # Test call flow with good options
    def test_zmq_client_configure_default(self):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            self.zmqd.configure()
            es_logger_default = ' '.join(
                es_logger.EsLogger.list_plugins(True, ['console_log_processor']))
            nose.tools.ok_(self.zmqd.process_console_logs == es_logger_default,
                           "{} did not match {}".format(self.zmqd.process_console_logs,
                                                        es_logger_default))
            es_logger_default = ' '.join(
                es_logger.EsLogger.list_plugins(True, ['gather_build_data']))
            nose.tools.ok_(self.zmqd.gather_build_data == es_logger_default,
                           "{} did not match {}".format(self.zmqd.gather_build_data,
                                                        es_logger_default))
            es_logger_default = ' '.join(
                es_logger.EsLogger.list_plugins(True, ['event_generator']))
            nose.tools.ok_(self.zmqd.generate_events == es_logger_default,
                           "{} did not match {}".format(self.zmqd.generate_events,
                                                        es_logger_default))

    def test_zmq_client_configure_generate_events_plugin_config(self):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            self.set_plugin_config(config)
            self.set_generate_events_plugin_config(config)
            self.zmqd.configure(config)
            for key in self.zmqd.env_vars:
                if hasattr(self.zmqd, key.lower()):
                    nose.tools.ok_(os.environ[key] == getattr(self.zmqd, key.lower()))
            for key in config['generate_events:plugin-a'].keys():
                nose.tools.ok_(os.environ[key.upper()] == config['generate_events:plugin-a'][key],
                               "Expected: {}, Actual: {}".format(
                                   config['generate_events:plugin-a'][key],
                                   os.environ[key.upper()]))
            for key in config['generate_events:pluginb'].keys():
                nose.tools.ok_(os.environ[key.upper()] == config['generate_events:pluginb'][key],
                               "Expected: {}, Actual: {}".format(
                                   config['generate_events:pluginb'][key],
                                   os.environ[key.upper()]))

    def test_zmq_client_configure_gather_build_data_plugin_config(self):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            self.set_plugin_config(config)
            self.set_gather_build_data_plugin_config(config)
            self.zmqd.configure(config)
            for key in self.zmqd.env_vars:
                if hasattr(self.zmqd, key.lower()):
                    nose.tools.ok_(os.environ[key] == getattr(self.zmqd, key.lower()))
            for key in config['gather_build_data:plugin'].keys():
                nose.tools.ok_(os.environ[key.upper()] == config['gather_build_data:plugin'][key],
                               "Expected: {}, Actual: {}".format(
                                   config['gather_build_data:plugin'][key],
                                   os.environ[key.upper()]))

    def test_zmq_client_configure_process_console_logs_plugin_config(self):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            self.set_plugin_config(config)
            self.set_process_console_logs_plugin_config(config)
            self.zmqd.configure(config)
            for key in self.zmqd.env_vars:
                if hasattr(self.zmqd, key.lower()):
                    nose.tools.ok_(os.environ[key] == getattr(self.zmqd, key.lower()))
            for key in config['process_console_logs:pluginA'].keys():
                nose.tools.ok_(
                    os.environ[key.upper()] == config['process_console_logs:pluginA'][key],
                    "Expected: {}, Actual: {}".format(
                        config['process_console_logs:pluginA'][key],
                        os.environ[key.upper()]))

    # Test call flow with good options
    def test_zmq_client_configure(self):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            self.set_plugin_config(config)
            self.zmqd.configure()
            for key in self.zmqd.env_vars:
                if hasattr(self.zmqd, key.lower()):
                    nose.tools.ok_(os.environ[key] == getattr(self.zmqd, key.lower()))

    @nose.tools.raises(es_logger.zmq_client.ZMQClientMisconfiguration)
    def test_zmq_missing_config(self):
        mock_args = unittest.mock.MagicMock()
        mock_args.test_zmq = False
        es_logger.zmq_client.ESLoggerZMQDaemon(mock_args).configure()

    def test_get_project_name(self):
        project = es_logger.zmq_client.ESLoggerZMQDaemon.get_project_name("//job/foo/job/bar/")
        nose.tools.ok_(project == '/foo/bar')

    def test_test_zmq_task(self):
        with nose.tools.assert_logs(level='DEBUG') as cm:
            self.zmqd.test_zmq_task([self.sample_finished_message])
        nose.tools.assert_equal(
            cm.output,
            ['INFO:root:Got event: job [folder/sample-job] number [123] phase [FINISHED]'])

    @unittest.mock.patch('es_logger.EsLogger', autospec=True)
    def test_es_logger_task(self, mock_esl):
        self.zmqd.jenkins_url = 'https://joc.example.com/jenkins/jenkins_url'
        mock_esl_instance = mock_esl(32500, ['logstash'])
        mock_esl_instance.post_all.return_value = 0
        with nose.tools.assert_logs(level='DEBUG') as cm:
            self.zmqd.es_logger_task([self.sample_finished_message])
        nose.tools.assert_equal(cm.output,
                                ['INFO:root:Process folder/sample-job number 123 on ' +
                                 'https://joc.example.com/jenkins/jenkins_url',
                                 'INFO:root:folder/sample-job number 123 status 0'])
        esl_calls = [unittest.mock.call().gather_all(),
                     unittest.mock.call().post_all()]
        nose.tools.assert_equals(mock_esl.method_calls, esl_calls)

    @unittest.mock.patch('es_logger.EsLogger', autospec=True)
    def test_es_logger_task_not_finished(self, mock_esl):
        self.zmqd.jenkins_url = 'https://joc.example.com/jenkins/jenkins_url'
        mock_esl_instance = mock_esl(32500, ['logstash'])
        mock_esl_instance.post_all.return_value = 0
        with nose.tools.assert_logs(level='DEBUG') as cm:
            self.zmqd.es_logger_task([self.sample_completed_message])
        nose.tools.assert_equal(cm.output,
                                ['DEBUG:root:Not collecting from job in phase COMPLETED'])
        esl_calls = []
        nose.tools.assert_equals(mock_esl.method_calls, esl_calls)

    def test_worker(self):
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status_list = asyncio.run(self.async_worker())
        nose.tools.ok_(self.zmqd.queue.qsize() == 0)
        nose.tools.ok_(status_list == [0])
        self.zmqd.es_logger_task.assert_called_with(self.sample_finished_message)
        print(cm.output)
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:worker-1 Starting',
             'DEBUG:root:worker-1 waiting for work',
             'DEBUG:root:worker-1 processing msg ' + self.sample_finished_message_log,
             'DEBUG:root:worker-1 result 0',
             'DEBUG:root:worker-1 waiting for work',
             'INFO:root:worker-1 cancelled, finishing',
             'INFO:root:worker-1 Finished'])

    async def async_worker(self):
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.es_logger_task = unittest.mock.MagicMock()
        self.zmqd.es_logger_task.return_value = 0
        task = asyncio.create_task(self.zmqd.worker('worker-1', 'es_logger_task'))
        self.zmqd.queue.put_nowait(self.sample_finished_message)
        # Yield control to the worker task
        await asyncio.sleep(1)
        # Cancel the worker
        task.cancel()
        return await asyncio.gather(task, return_exceptions=True)

    def test_worker_timeout(self):
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status_list = asyncio.run(self.async_worker_timeout())
        nose.tools.ok_(self.zmqd.queue.qsize() == 0)
        nose.tools.ok_(status_list == [0])
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:worker-1 Starting',
             'DEBUG:root:worker-1 waiting for work',
             'DEBUG:root:worker-1 timeout waiting for work, looping',
             'DEBUG:root:worker-1 waiting for work',
             'INFO:root:worker-1 cancelled, finishing',
             'INFO:root:worker-1 Finished'])

    async def async_worker_timeout(self):
        self.zmqd.queue = asyncio.Queue()
        task = asyncio.create_task(self.zmqd.worker('worker-1', 'test_zmq_task'))
        # Yield control to the worker task
        await asyncio.sleep(self.zmqd.worker_sleep + 1)
        # Cancel the worker
        task.cancel()
        return await asyncio.gather(task, return_exceptions=True)

    def test_drain_queue(self):
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status_list = asyncio.run(self.async_drain_queue())
        nose.tools.ok_(status_list == [0])
        calls = [unittest.mock.call(self.sample_completed_message),
                 unittest.mock.call(self.sample_finished_message)]
        self.zmqd.es_logger_task.assert_has_calls(calls)
        print(cm.output)
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Draining queue of size 2',
             'DEBUG:root:Processing msg ' + self.sample_completed_message_log,
             'DEBUG:root:Result None',
             'DEBUG:root:Processing msg ' + self.sample_finished_message_log,
             'DEBUG:root:Result 0',
             'INFO:root:Queue drained, processed 2'])

    async def async_drain_queue(self):
        self.zmqd.es_logger_task = unittest.mock.MagicMock()
        self.zmqd.es_logger_task.side_effect = [None, 0]
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.queue.put_nowait(self.sample_completed_message)
        self.zmqd.queue.put_nowait(self.sample_finished_message)
        return await self.zmqd.drain_queue()

    @unittest.mock.patch('es_logger.zmq_client.Context', autospec=True)
    def test_recv(self, mock_context):
        self.zmqd.zmq_publisher = 'tcp://jenkins.example.com:8888'
        mc_instance = mock_context.instance()
        self.zmqd.queue = asyncio.Queue()

        with nose.tools.assert_logs(level='DEBUG') as cm:
            listener = asyncio.run(self.async_recv(mc_instance))

        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Listener Starting against tcp://jenkins.example.com:8888',
             'DEBUG:root:Listener waiting for message',
             'DEBUG:root:Adding 1 to queue 0',
             'DEBUG:root:Listener waiting for message',
             'INFO:root:Listener cancelled, finishing',
             'INFO:root:Listener Finished'])
        context_calls = [
            unittest.mock.call.instance(),
            unittest.mock.call.instance().socket(),
            unittest.mock.call.instance(),
            unittest.mock.call.instance().socket(2),
            unittest.mock.call.instance().socket().connect('tcp://jenkins.example.com:8888'),
            unittest.mock.call.instance().socket().subscribe(b'')]
        mock_context.assert_has_calls(context_calls)
        nose.tools.ok_(listener.done())

    async def async_recv(self, mc_instance):
        future_msg = asyncio.Future()
        future_msg.set_result(1)
        mc_instance.socket().recv_multipart.side_effect = [future_msg, asyncio.Future()]
        listener = asyncio.create_task(self.zmqd.recv())
        await asyncio.sleep(1)
        listener.cancel()
        return listener

    def test_test_zmq_start(self):
        dummy_task = unittest.mock.MagicMock()
        dummy_task.return_value = self.dummyTask(0)
        self.zmqd.recv = dummy_task
        self.zmqd.worker = dummy_task
        self.zmqd.num_workers = 2
        with nose.tools.assert_logs(level='DEBUG') as cm:
            asyncio.run(self.async_start())
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Starting',
             'INFO:root:Started 2 workers using test_zmq_task'])
        nose.tools.ok_(len(self.zmqd.tasks) == 2)
        nose.tools.ok_(self.zmqd.listener)

    def test_es_logger_start(self):
        dummy_task = unittest.mock.MagicMock()
        dummy_task.return_value = self.dummyTask(0)
        self.zmqd.recv = dummy_task
        self.zmqd.worker = dummy_task
        self.zmqd.num_workers = 2
        self.zmqd.test_zmq = False
        with nose.tools.assert_logs(level='DEBUG') as cm:
            asyncio.run(self.async_start())
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Starting',
             'INFO:root:Started 2 workers using es_logger_task'])
        nose.tools.ok_(len(self.zmqd.tasks) == 2)
        nose.tools.ok_(self.zmqd.listener)

    async def async_start(self):
        self.zmqd.start()

    def test_stop(self):
        with nose.tools.assert_logs(level='DEBUG') as cm:
            asyncio.run(self.async_stop())
        # Use the log messages to assert we cancelled the tasks
        nose.tools.assert_regex(
            '\n'.join(cm.output),
            '\n'.join(
                [r'DEBUG:asyncio:Using selector: EpollSelector',
                 r'INFO:root:Stopping listener',
                 r'INFO:root:Stopping tasks',
                 r'DEBUG:root:All Tasks: {.*}',
                 r'INFO:root:Stopped, waiting for tasks to finish']))

    async def async_stop(self):
        self.zmqd.listener = asyncio.create_task(self.dummyTask(1, sleep=5))
        self.zmqd.tasks = [asyncio.create_task(self.dummyTask(2, sleep=5))]
        return self.zmqd.stop()

    # Tested as part of next 2 tests
    def test_check_task(self):
        pass

    def test_check_listener(self):
        self.zmqd.listener = unittest.mock.create_autospec(asyncio.Task)
        self.zmqd.listener.done = unittest.mock.Mock(side_effect=[False, True])
        task_status = self.zmqd.check_listener()
        nose.tools.ok_(task_status)
        task_status = self.zmqd.check_listener()
        nose.tools.ok_(not task_status)

    def test_check_tasks(self):
        self.zmqd.tasks = [unittest.mock.create_autospec(asyncio.Task),
                           unittest.mock.create_autospec(asyncio.Task)]
        self.zmqd.tasks[0].done = unittest.mock.Mock(side_effect=[False, False, True])
        self.zmqd.tasks[1].done = unittest.mock.Mock(side_effect=[False, True])
        task_status = self.zmqd.check_tasks()
        nose.tools.ok_(task_status)
        task_status = self.zmqd.check_tasks()
        nose.tools.ok_(not task_status)
        task_status = self.zmqd.check_tasks()
        nose.tools.ok_(not task_status)

    def test_all_tasks(self):
        self.zmqd.listener = unittest.mock.create_autospec(asyncio.Task)
        self.zmqd.tasks = [unittest.mock.create_autospec(asyncio.Task)]
        nose.tools.assert_equal(self.zmqd.all_tasks(), [self.zmqd.listener] + self.zmqd.tasks)

    def test_tasks_done(self):
        self.zmqd.listener = unittest.mock.create_autospec(asyncio.Task)
        self.zmqd.tasks = [unittest.mock.create_autospec(asyncio.Task)]
        self.zmqd.listener.done = unittest.mock.Mock(side_effect=[False, True, True])
        self.zmqd.tasks[0].done = unittest.mock.Mock(side_effect=[False, True])
        # No tasks done
        tasks_done = self.zmqd.tasks_done()
        nose.tools.ok_(not tasks_done)
        # One task done
        tasks_done = self.zmqd.tasks_done()
        nose.tools.ok_(not tasks_done)
        # All tasks done
        tasks_done = self.zmqd.tasks_done()
        nose.tools.ok_(tasks_done)

    def test_async_main(self):
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.check_listener = unittest.mock.MagicMock()
        self.zmqd.check_tasks = unittest.mock.MagicMock()
        self.zmqd.start = unittest.mock.MagicMock()
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status = asyncio.run(self.async_async_main())
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Started tasks, entering status check loop',
             'INFO:root:Exited status check loop',
             'INFO:root:Draining queue of size 0',
             'INFO:root:Queue drained, processed 0',
             'INFO:root:Gathering task statuses'])
        nose.tools.ok_(status == 0)

    async def async_async_main(self):
        self.zmqd.listener = asyncio.create_task(self.dummyTask(1, sleep=5))
        self.zmqd.tasks = [asyncio.create_task(self.dummyTask(2, sleep=5))]
        return await self.zmqd.async_main()

    def test_async_main_unknown(self):
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.start = unittest.mock.MagicMock()
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status = asyncio.run(self.async_async_main_unknown())
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Started tasks, entering status check loop',
             'INFO:root:Exited status check loop',
             'INFO:root:Draining queue of size 0',
             'INFO:root:Queue drained, processed 0',
             'INFO:root:Gathering task statuses',
             'WARNING:root:Unknown status: Bad return'])
        nose.tools.ok_(status == 1)

    async def async_async_main_unknown(self):
        self.zmqd.listener = asyncio.create_task(self.dummyTask(1))
        self.zmqd.tasks = [asyncio.create_task(self.dummyTask(2, return_status="Bad return"))]
        return await self.zmqd.async_main()

    def test_async_main_exception(self):
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.start = unittest.mock.MagicMock()
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status = asyncio.run(self.async_async_main_exception())
        nose.tools.assert_equal(
            cm.output,
            ['DEBUG:asyncio:Using selector: EpollSelector',
             'INFO:root:Started tasks, entering status check loop',
             'INFO:root:Exited status check loop',
             'INFO:root:Draining queue of size 0',
             'INFO:root:Queue drained, processed 0',
             'INFO:root:Gathering task statuses',
             'WARNING:root:Exception: Exception message'])
        nose.tools.ok_(status == 1)

    async def async_async_main_exception(self):
        self.zmqd.listener = asyncio.create_task(self.dummyTask(1))
        self.zmqd.tasks = [asyncio.create_task(
            self.dummyTask(2, exception=Exception("Exception message")))]
        return await self.zmqd.async_main()

    def test_async_main_bad_listener(self):
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.start = unittest.mock.MagicMock()
        self.zmqd.stop = unittest.mock.MagicMock()
        self.zmqd.check_listener = unittest.mock.MagicMock()
        self.zmqd.check_listener.return_value = False
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status = asyncio.run(self.async_async_bad_listener())
        nose.tools.ok_(status == 1)
        self.zmqd.stop.assert_called_once()
        nose.tools.assert_in('WARNING:root:Listener not running', cm.output)

    async def async_async_bad_listener(self):
        self.zmqd.listener = asyncio.create_task(
            self.dummyTask("listener", sleep=3, exception=Exception("Exception message")))
        self.zmqd.tasks = [asyncio.create_task(self.dummyTask("worker", sleep=5))]
        return await self.zmqd.async_main()

    def test_async_main_bad_worker(self):
        self.zmqd.queue = asyncio.Queue()
        self.zmqd.start = unittest.mock.MagicMock()
        self.zmqd.stop = unittest.mock.MagicMock()
        self.zmqd.check_tasks = unittest.mock.MagicMock()
        self.zmqd.check_tasks.return_value = False
        with nose.tools.assert_logs(level='DEBUG') as cm:
            status = asyncio.run(self.async_async_bad_worker())
        nose.tools.ok_(status == 1)
        self.zmqd.stop.assert_called_once()
        nose.tools.assert_in('WARNING:root:Tasks not running', cm.output)

    async def async_async_bad_worker(self):
        self.zmqd.listener = asyncio.create_task(self.dummyTask("listener", sleep=5))
        self.zmqd.tasks = [asyncio.create_task(
            self.dummyTask("worker", sleep=3, exception=Exception("Exception message")))]
        return await self.zmqd.async_main()

    @unittest.mock.patch('es_logger.zmq_client.asyncio', autospec=True)
    def test_main(self, mock_asyncio):
        with unittest.mock.patch('configparser.ConfigParser') as mock_config_parser:
            config = self.config_setup(mock_config_parser)
            self.set_default_config(config)
            self.set_plugin_config(config)
            mock_asyncio.run.return_value = 0
            self.zmqd.async_main = unittest.mock.Mock()
            status = self.zmqd.main()
        print(mock_asyncio.mock_calls)
        mock_asyncio.assert_has_calls([unittest.mock.call.run(self.zmqd.async_main())])
        nose.tools.assert_equal(status, 0)



# Copyright (c) 2018 Cisco Systems, Inc.
# All rights reserved.

__author__ = 'jonpsull'

import es_logger.interface
import nose


class DummyConsoleLogProcessor(es_logger.interface.ConsoleLogProcessor):
    def __init__(self):
        super().__init__()

    def process(self, console_log):
        return {'DummyConsoleLogProcessor': True}


class DummyGatherBuildData(es_logger.interface.GatherBuildData):
    def __init__(self):
        super().__init__()

    def gather(self, es_logger):
        return {'DummyGatherBuildData': True}


class DummyEventGenerator(es_logger.interface.EventGenerator):
    def __init__(self):
        super().__init__()

    def generate_events(self, es_logger):
        return [{'DummyEventGenerator': 1}, {'DummyEventGenerator': 2}]


class DummyEventTarget(es_logger.interface.EventTarget):
    def __init__(self):
        super().__init__()

    @staticmethod
    def get_help_string():
        return '''
Dummy Target Environment Variables:
    DUMMY_SERVER            The server to send events to
'''

    def validate(self):
        return True

    def send_event(self, json_event):
        if json_event is None:
            return 1
        return 0


class TestPlugins(object):

    def test_console_log_processor_plugins(self):
        clp = DummyConsoleLogProcessor()
        clp.process('log')

    def test_gather_build_data_plugins(self):
        gbd = DummyGatherBuildData()
        gbd.gather('log')

    def test_event_generator_plugins(self):
        eg = DummyEventGenerator()
        fields = eg.get_fields()
        nose.tools.ok_(fields == es_logger.interface.EventGenerator.DEFAULT_FIELDS)
        eg.generate_events({'es_logger': True})

    def test_event_target_plugins(self):
        help_str = DummyEventTarget.get_help_string()
        nose.tools.assert_is_instance(help_str, str)
        et = DummyEventTarget()
        et.send_event({'es_logger': True})
        et.finish_send()



__author__ = 'jonpsull'

from ..interface import EventGenerator
import logging

LOGGER = logging.getLogger(__name__)


class JUnitEvent(EventGenerator):
    """
    Process the junit output
    """
    def get_fields(self):
        return super(JUnitEvent, self).get_fields() + ['GERRIT_PATCHSET_REVISION',
                                                       'GERRIT_REFSPEC']

    def generate_events(self, esl):
        """
        return a list of objects to send as events

        :param esl: the calling es_logger object
        :type console_log: object
        :returns: list(dict(str:?))
        """
        LOGGER.debug("Starting: {}".format(type(self).__name__))
        output_list = []

        # Get the test report from Jenkins (if not already cached)
        test_report = esl.get_test_report()
        if test_report is not None:
            # Annotate the return with extra info
            test_report['type'] = 'total'
            test_report['totalCount'] = test_report['failCount'] +\
                test_report['skipCount'] + test_report['passCount']

            # Remove the suites, and process them as individual events
            test_suites = test_report.pop("suites")
            # Iterate and add 1 event per suite
            for suite in test_suites:
                test_cases = suite.pop("cases")
                suite_name = suite['name']
                suite_pass = 0
                suite_skip = 0
                suite_fail = 0
                suite_unknown = 0
                # Iterate and add 1 event per case
                for case in test_cases:
                    case['suite'] = suite_name
                    if case['status'] == 'PASSED':
                        suite_pass += 1
                    elif case['status'] == 'SKIPPED':
                        suite_skip += 1
                    elif case['status'] == 'FAILED':
                        suite_fail += 1
                    else:
                        suite_unknown += 1
                    case['type'] = 'case'
                    if 'errorDetails' in case.keys():
                        if case['errorDetails'] is not None:
                            case['errorDetailsTruncated'] = case['errorDetails'][:255]
                        else:
                            case['errorDetailsTruncated'] = None
                    output_list.append(case)
                suite['passCount'] = suite_pass
                suite['skipCount'] = suite_skip
                suite['failCount'] = suite_fail
                suite['unknownCount'] = suite_unknown
                suite['totalCount'] = suite_pass + suite_skip + suite_fail + suite_unknown
                suite['type'] = 'suite'
                output_list.append(suite)

            # Add our overall result as an event
            output_list.append(test_report)
        LOGGER.debug("Finished: {}".format(type(self).__name__))
        return output_list
